-- Copyright (c) 2015 Algo Communication Products Ltd.

require('conf')

local env_cache = '/tmp/web/session_env.lua'

local redirect_tmpl = [[<html><head>
<meta http-equiv="refresh" content="0;url=%s" /></head></html>]]

function session_start_page()
	mg.write('HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n')
end

function session_redirect(url)
	session_start_page()
	mg.write(string.format(redirect_tmpl, url))
end

function session_reset()
	session_redirect('/')
end

function session_reboot()
	session_redirect('/control/reboot')
end

function session_load()
	local cookie_header = mg.request_info.http_headers.Cookie
	local req_cookie
	if cookie_header then
		req_cookie = cookie_header:gmatch('id=(%d+)')()
	end

	dofile(env_cache)
	if g_cookie == req_cookie then
		local now = os.time()
		local timeout = conf_getpn('admin.web.timeout')
		if (timeout == 0 or (now - g_timer) < timeout) then
			return 0
		else
			session_destroy()
			return 1
		end
	else
		return 1
	end
end

function session_generate()
	local timer, cookie
	timer = os.time()
	math.randomseed(timer)
	cookie = tostring(math.random(2147483647))	-- 2^31-1
	local content = string.format('g_cookie = "%s"\ng_timer = %d\n',
		cookie, timer)
	local file, err = io.open(env_cache, 'w')
	if not err then
		file:write(content)
		file:close()
	end
	mg.write('HTTP/1.0 200 OK\r\n')
	mg.write(string.format('Set-Cookie: id=%d;path=/;\r\n', cookie))
	mg.write('Content-Type: text/html\r\n\r\n')
	mg.write(string.format(redirect_tmpl, prod_startpage))
end

function session_save()
	if not g_cookie then g_cookie = 'loggedout' end
	if not g_reboot then g_reboot = 0 end
	local now = os.time()
	local file, err = io.open(env_cache, 'w')
	if not err then
		file:write(string.format(
			'g_cookie = "%s"\ng_reboot = %d\ng_timer = %d\n',
			g_cookie, g_reboot, now))
		file:close()
	end
end

function session_destroy()
	local file, err = io.open(env_cache, 'w')
	if not err then
		file:write('g_cookie = "loggedout"\n')
		file:close()
	end
end

function session_send_file(fn)
	local file, err = io.open(fn, 'rb')
	if not err then
		while true do
			local block = file:read(1300)
			if not block then break end
			mg.write(block)
		end
		file:close()
	end
end

function session_save_file(fn)
	local file, err = io.open(fn, 'wb')
	if not err then
		file:write(mg.read())
		file:close()
	end
end

function session_download(fn)
	mg.write('HTTP/1.0 200 OK\r\nContent-Type: application/octet-stream\r\n\r\n')
	session_send_file(fn)
end
