-- Copyright (c) 2008-2016 Algo Communication Products Ltd.

require('conf')
require("urlcode")
require("readuntil")
require("post")

local header_tmpl = [[<!DOCTYPE html>
<html><head>
%s%s%s%s<title>%s Control Panel</title></head>
<body>
<div id="container">
<div id="header">
<table id="header"><tr>
<td id="logo"><img src="%s" alt="logo" /></td>
<td id="conn"></td>
<td id="hmid"><text id="prodname">%s</text></td>
<td id="info">Firmware: %s</td></tr>
</table></div>
<div id="%s">]]

webpage_footer = [[
</div>
<div id="footer">&copy; Copyright 2016 Algo Communication Products Ltd.</div>
</body></html>]]

webpage_mc_ft = [[
</div></div></div>]]

function webpage_read_file(filename, default)
	local file, err = io.open(filename, 'r')
	local status
	if err then
		status = default
	else
		status = file:read('*all')
		file:close()
	end
	return status
end

function webpage_write_file(filename, content)
	local file, err = io.open(filename, 'w')
	if not err then
		status = file:write(content)
		file:close()
	end
end

function webpage_cst_header(pub, title, header, fav, logo, css, js, fixedwidth)
	local version = webpage_read_file('/opt/algo/info/firmware-version', 'Unknown')

	if not title then
		title = conf_getdevice('product.vendor') .. ' ' ..
			conf_getdevice('product.name')
	end

	if not header then
		header = conf_getdevice('product.name') .. ' ' ..
			conf_getdevice('product.desc') .. ' Control Panel'
	end

	local fav_tag
	if not fav then
		fav = '/static/favicon.ico'
	end
	if fav == '' then
		fav_tag = ''
	else
		fav_tag = string.format('<link rel="shortcut icon" type="image/x-icon" href="%s?v=' ..
			version .. '">\n', fav)
	end

	local css_tag = ''
	if not css then css = {'/static/lanbox.css'} end
	for i, v in ipairs(css) do
		css_tag = css_tag .. string.format('<link rel="stylesheet" type="text/css" href="%s?v=' ..
			version .. '">\n', v)
	end

	local js_tag = ''
	if not js then js = {'/static/lanbox.js'} end
	for i, v in ipairs(js) do
		js_tag = js_tag .. string.format('<script type="text/javascript" src="%s?v=' ..
			version .. '"></script>\n', v)
	end

	if not logo then logo = '/static/logo.png' end

	local refresh_tag = ''
	if not pub then
		local timeout = conf_getpn('admin.web.timeout')
		if timeout > 0 then
			refresh_tag = string.format('<meta http-equiv="refresh" content="%d;url=/" />\n', timeout + 1);
		end
	end

	local content_id
	if fixedwidth == nil then
		content_id = 'content'
	else
		content_id = 'content-fixed'
	end

	return string.format(header_tmpl, refresh_tag, fav_tag, css_tag, js_tag,
		title, logo, header, version, content_id)
end

local menu_hd = [[<div id='tabmenu'><div class='tabmenu1'>
<ul class='tabmenu l1'>]]
local menu_top_logout = [[
<li class="tabmenu-item-logout"><a href="/control/logout.lua">Logout</a></li>]]
local menu_item_tmpl = [[<li%s><a href="%s">%s</a></li>]]
local menu_mid = [[</ul><br style="clear:both" /><div class="tabmenu2">
<ul class="tabmenu l2">]]
local menu_tmpl = [[<li><a href="%s">%s</a></li>]]
local menu_ft = [[</ul><br style="clear:both" /></div></div></div>]]
local menu_ft_nosecond = [[</ul></div></div></div>]]

local function cst_list(desc, active_item)
	local list_html = ''
	for i, item in ipairs(desc[1]) do
		if active_item == item then
			active = ' class="active"'
		else
			active = ''
		end
		list_html = list_html .. string.format(menu_item_tmpl, active,
			desc[2][i], item)
	end
	return list_html
end

function webpage_cst_menu(menu_top, menu_second, top, second, pub)
	local logout_item
	if pub then
		logout_item = ''
	else
		logout_item = menu_top_logout
	end
	if second then
		return menu_hd .. cst_list(menu_top, top) ..
			logout_item .. menu_mid ..
			cst_list(menu_second[top], second) .. menu_ft
	else
		return menu_hd .. cst_list(menu_top, top) ..
			logout_item .. menu_mid .. menu_ft_nosecond
	end
end

local mchd_start = [[
<div id="maincontent">
<div id="banner">
<noscript><div class="errorbox"><strong>Java Script</strong><br />
You must enable Java Script in your browser or the web interface will not work properly.</div></noscript>]]
local mchd_reboot_msg = [[
<div class="errorbox">
<form method="post" name="cbi" action="/control/reboot.lua">
<strong>Reboot Required</strong><br />
Please reboot the device to apply all the settings.
<input type="submit" class="cbi-button cbi-button-reset" name="reboot"
 value="Reboot" /></form></div>]]
local mchd_saved_msg = [[
<div class="notifbox">
<strong>Saved</strong><br />
Setting changes have been saved and will be applied on the next call.</div>
]]
local mchd_msg_tmpl = [[
<div class="errorbox"><strong>%s</strong><br />%s</div>]]
function webpage_cst_mchd_app(saved, reboot, extra, js)
	local ret = mchd_start

	if g_reboot == 1 or reboot == 1 then
		ret = ret .. mchd_reboot_msg
	else
		if saved == 1 then
			ret = ret .. mchd_saved_msg
		end
	end

	if extra then
		for k,v in pairs(extra) do
			ret = ret .. string.format(mchd_msg_tmpl, k, v)
		end
	end

	if js then
		ret = ret .. js
	end

	return ret .. '</div>'	--banner
end

webpage_mcft_app = [[</div></div>]]

function webpage_cst_mchd(saved, reboot, extra, js)
	return webpage_cst_mchd_app(saved, reboot, extra, js) ..
		'<div id="paramcontent">'
end

local welcometitle_tmpl = [[
<h2>Welcome to the %s %s %s Control Panel</h2>]]
function webpage_cst_welcometitle(vendor, prodid, prodname)
	return string.format(welcometitle_tmpl, vendor, prodid, prodname)
end

local welcome_tmpl = [[
<div class="cbi-map-descr">
<p>Setting up your %s:</p>
<p><b>Step 1: Configure your %s</b></p>
<p>Log in with the default password and use the Basic Settings pages to set up the basic information.</p>
<p><b>Step 2: Check network settings (Optional)</b></p>
<p>Use the Network page under the Advanced Settings tab to change network settings.  The default setting for the device is to obtain its IP address from a DHCP server.
Contact your Network System administrator if you plan to assign a static IP address, Mask, and Gateway to the device.</p>
<p><b>Step 3: Secure your %s (Optional)</b></p>
<p>Use the Admin page under the Advanced Settings tab to change the administrator password.<br>
<img src="/static/exclamation.png" />Changing the password is extremely important if the device is directly connected to a public network.</p>
]]
local reg_tmpl = [[
<p><b>Step 4: Register your %s (Optional)</b></p>
<p>Please register your product using the link below:</p>
<p><a href="http://www.algosolutions.com/register" target="_blank">http://www.algosolutions.com/register</a></p>
<p>Registration ensures your access to the latest upgrades to this product and important service notices.</p>]]
local welcome_end = [[</div>]]
function webpage_cst_welcome(company, prodid, prodname, reg)
	local reg_info

	if reg then
		reg_info = string.format(reg_tmpl, prodname)
	else
		reg_info = ''
	end
	return string.format(welcome_tmpl, prodname, prodname, prodname) ..
		reg_info .. welcome_end
end

function webpage_get_network()
	local netif
	local vlan_inuse = conf_getpn('net.vlan.use')
	local vlan_id = conf_getpn('net.vlan.id')
	if vlan_inuse == 0 then
		netif = 'eth0'
	else
		netif = 'eth0.' .. vlan_id
	end
	local file, err = io.popen('ifconfig ' .. netif)
	if err then return '', '', '' end

	local line = file:read()
	local t = {}
	for id in line:gmatch('%S+') do
		table.insert(t, id)
	end
	local mac = t[5];
	line = file:read();
	file:close()
	t = {}
	for id in line:gmatch('%S+') do
		table.insert(t, id)
	end
	local ip = string.sub(t[2], 6)
	local netmask = string.sub(t[4], 6)
	return mac, ip, netmask
end

local status_tmpl = [[
<div class="cbi-value"><label class="cbi-value-title">%s</label>
<div class="cbi-value-field">%s</div></div>]]
function webpage_cst_status(label, desc)
	return string.format(status_tmpl, label, desc)
end

local form_start_tmpl = [[
<form method="post" action="%s">]]
function webpage_cst_formstart(url)
	return string.format(form_start_tmpl, url)
end

local page_start_tmpl = [[
<div class="cbi-map"><h2><a id="content">%s</a></h2>
<div class="cbi-map-descr">%s</div>]]
function webpage_cst_pagestart(title, desc)
	if desc == nil then desc = '' end
	return string.format(page_start_tmpl, title, desc)
end

local secthd_tmpl = [[
<fieldset class="cbi-section"%s%s><legend>%s</legend>]]
function webpage_cst_secthd(legend, id, hide, desc)
	local id_st, display

	if id then
		id_st = string.format(' id="%s"', id)
	else
		id_st = ''
	end
	if hide == 1 then
		display = ' style="display:none"'
	else
		display = ''
	end
	ret = string.format(secthd_tmpl, id_st, display, legend)
	if desc then
		ret = ret .. '<div class="cbi-section-descr">' .. desc ..
			'</div>'
	end
	return ret
end

webpage_subsect_hd = [[
<fieldset class="cbi-section-node">]]

function webpage_cst_subsect_hd(id, hide)
	local display
	if hide == 1 then
		display = ' style="display:none"'
	else
		display = ''
	end
	return string.format([[<fieldset class="cbi-section-node" id="%s" %s>]],
		id, display)
end

webpage_subsect_ft = [[
</fieldset><br />]]

webpage_sect_ft = [[
</fieldset>]]

webpage_page_end = [[
<br /></div>]]

webpage_form_end = [[
</form>]]

webpage_form_end_save = [[
<div class="cbi-page-actions">
<input class="cbi-button cbi-button-save" type="submit" value="Save" />
</div></form><div class="clear"></div>]]

local param_tmpl = [[
<div class="cbi-value"%s%s><label class="cbi-value-title">%s</label>
<div class="cbi-value-field">
%s%s
</div></div>]]
local desc_tmpl = [[
<br /><div class="cbi-value-description"><span class="cbi-value-helpicon">
%s</span>%s</div>
]]
--<img src="/static/information-white.png" alt="help" />
function webpage_cst_param(label, value_html, desc, id, hide, icon)
	local dest_st, id_st, display, icon_st

	if icon then
		icon_st = string.format('<img src="%s" alt="help" />', icon)
	else
		icon_st = '<img src="/static/information-white.png" alt="help" />'
	end
	if desc then
		desc_st = string.format(desc_tmpl, icon_st, desc)
	else
		desc_st = ''
	end
	if id then
		id_st = string.format(' id="%s"', id)
	else
		id_st = ''
	end
	if hide == 1 then
		display = ' style="display:none"'
	else
		display = ''
	end
	return string.format(param_tmpl, id_st, display, label, value_html,
		desc_st)
end

local param_tmpl_lesswhitespace = [[
<div class="cbi-value"%s%s><label class="cbi-value-title-lesswhitespace">%s</label>
<div class="cbi-value-field-lesswhitespace">
%s%s
</div></div>]]
function webpage_cst_param_lesswhitespace(label, value_html, desc, id, hide)
	local dest_st, id_st, display

	if desc then
		desc_st = string.format(desc_tmpl, desc)
	else
		desc_st = ''
	end
	if id then
		id_st = string.format(' id="%s"', id)
	else
		id_st = ''
	end
	if hide == 1 then
		display = ' style="display:none"'
	else
		display = ''
	end
	return string.format(param_tmpl_lesswhitespace, id_st, display, label, value_html,
		desc_st)
end

local textarea_tmpl = [[
<div %s%s>
<textarea style="width:100%%;height:20em" name="input">%s</textarea>%s</div>]]
function webpage_cst_textarea(value, desc, id, hide)
	local dest_st, id_st, display

	if desc then
		desc_st = string.format(desc_tmpl, desc)
	else
		desc_st = ''
	end
	if id then
		id_st = string.format(' id="%s"', id)
	else
		id_st = ''
	end
	if hide == 1 then
		display = ' style="display:none"'
	else
		display = ''
	end
	return string.format(textarea_tmpl, id_st, display, value, desc_st)
end

local column_tmpl = [[
<label class="columnheader"%s>&nbsp;%s</label>
]]
function webpage_cst_columnheader(value, id)
	local id_st
	if id then
		id_st = string.format(' id="%s"', id)
	else
		id_st = ''
	end
	return string.format(column_tmpl, id_st, value)
end

local input_text_tmpl = [[
<input type="text" class="cbi-input-text" name="%s" value="%s"%s/>]]
function webpage_cst_text(name, onchange, default)
	local js
	local value = conf_getp(name)
	if onchange then
		js = ' onchange="' .. onchange .. '"'
	else
		js = ''
	end
	if default ~= nil then
		if value == nil or value == '' then
			value = default
		end
	end
	return string.format(input_text_tmpl, name, value, js)
end

local pwd_tmpl = [[
<div class="cbi-value"%s%s><label class="cbi-value-title">%s</label>
<div class="cbi-value-field">
<input type="password" class="cbi-input-password" onchange="cbi_d_update(this.id)" name="%s" id="%s" value="%s" />
<img src="/static/arrow-circle-double.png" style="vertical-align:middle" title="Reveal/hide password" onclick="var e = document.getElementById('%s'); e.type = (e.type=='password') ? 'text' : 'password';" />
</div></div>]]
function webpage_cst_pwd(label, name, id, hide)
	local id_st, display

	if id then
		id_st = string.format(' id="%s"', id)
	else
		id_st = ''
	end
	if hide == 1 then
		display = ' style="display:none"'
	else
		display = ''
	end
	return string.format(pwd_tmpl, id_st, display, label, name, name,
		conf_getp(name), name)
end

local radio_tmpl = [[<span%s%s>%s<input type="radio" value="%s" name="%s" %s%s>%s</input></span>]]
function webpage_cst_radio(name, desc, value, onclick, binary, hide, class,
	valign, default)
	local i, v
	local rv, flag, js
	local pval = conf_getp(name)
	local ret = ''
	local display, br

	if (binary or not value) and pval == '' then
		pval = '0'
	end
	for i, v in ipairs(desc) do
		if value then
			rv = value[i]
		else
			rv = tostring(i - 1)
		end

		if pval == rv or default == rv then
			flag = ' checked'
		else
			flag = ''
		end

		if onclick and onclick[i] then
			js = ' onclick="' .. onclick[i] .. '"'
		else
			js = ''
		end

		if hide and hide[i] == 1 then
			display = ' style="display:none"'
		else
			display = ''
		end

		if class and class[i] then
			c = ' class="' .. class[i] .. '"'
		else 
			c = ''
		end

		if i > 1 and valign then
			br = '<br>'
		else
			br = ''
		end
		ret = ret .. string.format(radio_tmpl, display, c, br, rv, name,
			flag, js, v)
	end
	return ret
end

function webpage_cst_enabled(name, onclick)
	return webpage_cst_radio(name, {'Enabled', 'Disabled'}, {'1', '0'},
		onclick, true)
end

function webpage_cst_yesno(name, onclick)
	return webpage_cst_radio(name, {'Yes', 'No'}, {'1', '0'},
		onclick, true)
end

function webpage_cst_onoff(name, onclick)
	return webpage_cst_radio(name, {'On', 'Off'}, {'1', '0'},
		onclick, true)
end

local opt_tmpl = [[<option value="%s"%s>%s</option>]]
function webpage_cst_select(name, desc, value, isnum, hide, class, default)
	local i, v
	local rv, flag
	local pval = conf_getp(name)
	local opts = ''
	local found = 0
	local vis
	local attr_class

	if isnum and pval == '' then
		pval = '0'
	end

	for i, v in ipairs(desc) do
		if value then
			rv = value[i]
		else
			rv = tostring(i - 1)
		end
		if pval == rv or default == rv then
			flag = ' selected'
			found = 1
		else
			flag = ''
		end
		opts = opts .. string.format(opt_tmpl, rv, flag, v)
	end
	if sel then
		opts = opts .. string.format(opt_tmpl, sel, ' selected', sel)
		found = 1
	end
	if found == 0 then
		opts = opts .. string.format(opt_tmpl, pval,
			' selected', pval .. ' (?)')
	end

	if hide == 1 then
		display = ' style="display:none"'
	else
		display = ''
	end

	if class then
		attr_class = ' class="' .. class .. '"'
	else
		attr_class = ''
	end

	return '<select name="' .. name .. '"' .. display .. attr_class ..
		'>' .. opts .. '</select>'
end

local button_tmpl = [[
<input type="%s" class="cbi-button cbi-button-%s" name="%s" value="%s"%s />]]
function webpage_cst_button(name, value, style, onclick, button)
	if button then
		itype = 'button'
	else
		itype = 'submit'
	end
	if onclick then
		js = string.format(' onClick="%s"', onclick)
	else
		js = ''
	end
	return string.format(button_tmpl, itype, style, name, value, js)
end

local upload_tmpl = [[<input type="file" name="%s" />]]
function webpage_cst_upload(name)
	return string.format(upload_tmpl, name)
end

function webpage_parse_post(len, settings)
	data = mg.read(len)
	cgilua.urlcode.parsequery(data, settings)
end

--[[
vars format:
{
	upgrade="Upgrade",
	md5={
		filesize=53,
		file=file (0xc397d08),
		filename="algo-8180-2.0.7.md5",
		["content-type"]="application/x-extension-md5"
	},
	firmware={
		filesize=1331200,
		file=file (0xc83ee80),
		filename="algo-8180-2.0.7.fw",
		["content-type"]="application/octet-stream"
	}
}
]]
function webpage_parse_multipart(len, vars)
	cgilua.post.parsedata {
		read = mg.read,
		content_type = mg.request_info.http_headers['Content-Type'],
		content_length = len,
		maxinput = 4 * 1024 * 1024,
		maxfilesize = 4 * 1024 * 1024,
		args = vars,
		dir = "/tmp/web/mp/",
	}
end

webpage_credit_info = [[
<style  TYPE="text/css">
/* http://users.tkk.fi/~tkarvine/pre-wrap-css3-mozilla-opera-ie.html */
pre {
 white-space: pre-wrap;       /* css-3 */
 white-space: -moz-pre-wrap;  /* Mozilla, since 1999 */
 white-space: -pre-wrap;      /* Opera 4-6 */
 white-space: -o-pre-wrap;    /* Opera 7 */
 word-wrap: break-word;       /* Internet Explorer 5.5+ */
}
</style>
<a href="http://kernel.org/">Linux Kernel</a>, License:<a href="http://www.gnu.org/licenses/gpl-2.0.txt">GPL 2.0</a>:
<pre>Copyright 1997-2013 The Linux Kernel Organization, Inc.
Distributed software is copyrighted by their respective contributors and are distributed under their own individual licenses.
Linux is a Registered Trademark of Linus Torvalds. All trademarks are property of their respective owners.</pre>
<a href="http://debian.org/">Debian</a>, License: Free software, mainly the GNU GPL, and other licenses:
<pre>Copyright (c) 1997-2014 SPI and others
Debian is a registered trademark of Software in the Public Interest, Inc.</pre>
Copyright information of all installed Debian packages can be downloaded from <a href="/control/copyright.txt">here</a>.<br>
<a href="http://www.denx.de/wiki/U-Boot/WebHome">U-Boot</a>, License:<a href="http://www.gnu.org/licenses/gpl.html">GPL</a>:
<pre>Copyright (c) 2002-2013 by DENX Software Engineering</pre>
<a href="https://github.com/bel2125/civetweb">Civetweb</a>, License:<a href="https://github.com/bel2125/civetweb/blob/master/LICENSE.md">MIT-style</a>:
<pre>Copyright (c) 2004-2013 Sergey Lyubka
Copyright (c) 2013 No Face Press, LLC (Thomas Davis)
Copyright (c) 2013 F-Secure Corporation</pre>
<a href="http://www.lua.org/">Lua</a>, License:<a href="http://www.lua.org/license.html">MIT-style</a>:
<pre>Copyright (c) 1994-2008 Lua.org, PUC-Rio. </pre>
<a href="http://www.keplerproject.org/cgilua/">CGILua</a>, License:<a href="http://www.keplerproject.org/cgilua/license.html">MIT-style</a>:
<pre>Copyright (c) 2003 Kepler Project.</pre>
<a href="http://luci.subsignal.org/trac">LuCI</a>, License: <a href="http://luci.subsignal.org/trac/wiki/License">Apache License 2.0</a>
<pre>Copyright 2008 Steven Barth <steven@midlink.org>
Copyright 2008 Jo-Philipp Wich <xm@leipzig.freifunk.net></pre>
<a href="http://www.soft-switch.org/">SpanDSP</a>, License:<a href="http://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html">LGPL 2.1</a>:
<pre>Copyright (c) 2001-2014 Steve Underwood <steveu@coppice.org></pre>
<a href="http://www.webrtc.org/">WebRTC</a>, License:<a href="http://www.webrtc.org/license-rights/license">BSD-style</a>:
<pre>Copyright (c) 2011, The WebRTC project authors. All rights reserved.</pre>
The source code can be downloaded from the web site listed above.  All the modifications can be obtained from Tech Support.<br>
<a href="http://pinvoke.com">Fugue Icons</a>, License:<a href="http://creativecommons.org/licenses/by/3.0/">Creative Commons Attribution 3.0 license</a>:
<pre>
Some icons by Yusuke Kamiyamane.  Copyright (c) 2015 Yusuke Kamiyamane. All rights reserved.
</pre>]]

local algo_addr = [[
Algo Communication Products Ltd.<br />
4500 Beedie Street<br />
Burnaby, BC  V5J 5L2<br />
Canada]]
local algo_support = [[
Email: support@algosolutions.com<br />
Web: www.algosolutions.com]]
function webpage_cst_algoinfo()
	return webpage_cst_secthd('Company Info') .. webpage_subsect_hd ..
		webpage_cst_status('Address', algo_addr) ..
		webpage_cst_status('Support', algo_support) ..
		webpage_subsect_ft .. webpage_sect_ft
end

function show_hwinfo()
	local prodid, hwver
	local file, err

	file, err = io.popen('/opt/algo/bin/fw_printenv -n algoprodid')
	if err then
		prodid = '8188'
	else
		prodid = file:read('*all')
		file:close()
	end

	file, err = io.popen('/opt/algo/bin/fw_printenv -n algohwver')
	if err then
		hwver = 'v1'
	else
		hwver = file:read('*all')
		file:close()
	end

	return prodid .. ' ' .. hwver
end

function webpage_cst_prodinfo(prodname, extra)
	local mac, ip, netmask = webpage_get_network()
	local info = webpage_cst_status('Product Name', prodname) ..
		webpage_cst_status('Firmware Version',
			webpage_read_file('/opt/algo/info/firmware-version',
				'Unknown')) ..
		webpage_cst_status('Base Version',
			webpage_read_file('/opt/algo/info/base-version',
				'Unknown')) ..
		webpage_cst_status('System Version',
			webpage_read_file('/opt/algo/info/sys-version',
				'Unknown')) ..
		webpage_cst_status('MAC Address', mac) ..
		webpage_cst_status('Hardware Info', show_hwinfo())
	if extra then
		for k, v in pairs(extra) do
			info = info .. webpage_cst_status(k, v)
		end
	end
	return info
end

function webpage_cst_fieldbutton(text, action)
	return string.format([[<label> </label><button type="button" class="field-button" onclick="%s">%s</button>]],
		action, text)
end

function webpage_cst_tonelist_values(desc, value, extra)
	for file in lfs.dir('/opt/algo/web/tones') do
		if file ~= '.' and file ~= '..' then
			table.insert(value, file)
		end
	end
	table.sort(value)

	for i, v in ipairs(value) do
		desc[i] = v
	end

	if extra then
		for k, v in pairs(extra) do
			table.insert(value, 1, k)
			table.insert(desc, 1, v)
		end
	end
end

function webpage_cst_tonelist(name, extra)
	local desc = {}
	local value = {}
	webpage_cst_tonelist_values(desc, value, extra)
	return webpage_cst_select(name, desc, value)
end
