-- Copyright (c) 2013-2016 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

if not g_reboot then g_reboot = 0 end

local flags = {['any'] = 0, ['pwd'] = 0, ['hostname'] = 0, ['reboot'] = 0,
	['log'] = 0}
local msg = {}

local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)

	if settings['admin.pwd'] == settings['pwd2'] then
		conf_update('admin.pwd', settings, flags, 'pwd')
	else
		msg['Admin Password Mismatch'] = 'Password was not saved.'
	end
	conf_update('admin.devname', settings, flags, 'hostname')
	conf_update('admin.startuptone', settings, flags, 'any')
	conf_update('admin.welcome', settings, flags, 'reboot')
	conf_update('admin.web.timeout', settings, flags, 'any')

	conf_update('log.level', settings, flags, 'log')
	conf_update('log.method', settings, flags, 'log')
	conf_update('log.server', settings, flags, 'log')

--	conf_update('watcher.watchdog.use', settings, flags, 'reboot')

	conf_update('net.srv.snmp', settings, flags, 'reboot')
	conf_update('net.http', settings, flags, 'reboot')

	if conf_getdevice('feature.synapps') == '1' then
		conf_update('synapps.use', settings, flags, 'reboot')
		conf_update('synapps.server', settings, flags, 'reboot')
		conf_update('synapps.port', settings, flags, 'reboot')
	end

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['pwd'] == 1 then
		os.execute('/opt/algo/bin/apply-settings pwd')
		flags['reboot'] = 1
	end
	if flags['hostname'] == 1 then
		os.execute('/opt/algo/bin/apply-settings hostname')
		flags['reboot'] = 1
	end
	if flags['log'] == 1 then
		os.execute('/opt/algo/bin/apply-settings log')
		flags['reboot'] = 1
	end
	if flags['reboot'] == 1 then
		g_reboot = 1
		session_save()
	end
end
session_save()

local form_start = webpage_cst_formstart('/control/admin.lua')

local page_start = webpage_cst_pagestart('Admin Settings')

secthd_pwd = webpage_cst_secthd('Admin Password')

local admin_pwd_tmpl = [[
<div class="cbi-value"><label class="cbi-value-title">Password</label>
<div class="cbi-value-field">
<input id="admin_pwd1" type="password" name="admin.pwd" value="%s"
 onchange="valPwd(this,'admin_pwd2','inf_admin_pwd', false);">
<img src="/static/arrow-circle-double.png" style="vertical-align:middle" title="Reveal/hide password" onclick="var e = document.getElementById('admin_pwd1'); e.type = (e.type=='password') ? 'text' : 'password';" />
</div></div>
<div class="cbi-value"><label class="cbi-value-title">Confirmation</label>
<div class="cbi-value-field">
<input id="admin_pwd2" type="password" name="pwd2" value="%s"
 onchange="valPwd(this,'admin_pwd1','inf_admin_pwd', false);">
<img src="/static/arrow-circle-double.png" style="vertical-align:middle" title="Reveal/hide password" onclick="var e = document.getElementById('admin_pwd2'); e.type = (e.type=='password') ? 'text' : 'password';" />
<img id="inf_admin_pwd" style="vertical-align:middle;display:none" src="/static/exclamation.png">
</div></div>
]]
local admin_pwd = string.format(admin_pwd_tmpl, conf_getp('admin.pwd'),
	conf_getp('admin.pwd'))

local secthd_gen = webpage_cst_secthd('General')

local p_dname = webpage_cst_param('Device Name (Hostname)',
	webpage_cst_text('admin.devname'))

local p_welcome = webpage_cst_param('Introduction Section on Status Page',
	webpage_cst_onoff('admin.welcome'))

local p_timeout = webpage_cst_param('Web Interface Session Timeout',
	webpage_cst_select('admin.web.timeout',
		{"Disabled", "5 minutes", "10 minutes", "15 minutes",
			"30 minutes", "1 hour"},
		{"0", "300", "600", "900", "1800", "3600"}),
	'Automatically log out web interface after period of inactivity.')

local tone_desc

if conf_getdevice('product.name') == '8301' or conf_getdevice('product.name') == '8373' then
	tone_desc = 'A tone can be played at startup to confirm that the device has booted. This can be useful when testing or configuring a device, but might not be desirable if the device is connected to an external amplifier and paging system.'
else
	tone_desc = 'A tone can be played at startup to confirm that the device has booted.'
end

local p_startuptone = webpage_cst_param('Play Tone at Startup',
	webpage_cst_enabled('admin.startuptone'), tone_desc)

local secthd_log = webpage_cst_secthd('Log Settings')

local level_desc = {'Error (Lowest)', 'Notice ("Event")',
	'Info ("SIP")', 'Debug (Highest)'}
local level_value = {'error', 'notice', 'info', 'debug'}
local p_log_level = webpage_cst_param('Log Level',
	webpage_cst_radio('log.level', level_desc, level_value))

local js_logm = [[
<script type="text/javascript">
function display_log_server(mode) {
	var display = (mode == 0) ? 'none' : 'block';
	document.getElementById("log_server").style.display = display;
}
</script>]]
local logm_js0 = [[display_log_server(0);]]
local logm_js1 = [[display_log_server(1);]]
local method_desc = {'Local', 'Network', 'Both'}
local method_value = {'local', 'network', 'both'}
local p_log_method = webpage_cst_param('Log Method',
	webpage_cst_radio('log.method', method_desc, method_value,
	{logm_js0, logm_js1, logm_js1}))
local hide
if conf_getp('log.method') == 'local' then hide = 1 else hide = 0 end
local p_log_server = webpage_cst_param('Log Server',
	webpage_cst_text('log.server'), nil, 'log_server', hide)

--[[
local secthd_ha = webpage_cst_secthd('High Availability')
local p_watchdog = webpage_cst_param('Hardware Watchdog',
	webpage_cst_enabled('watcher.watchdog.use'))
]]

local secthd_mgmt = webpage_cst_secthd('Management')

--[[
 Web Interface Protocol Options
 0: HTTP Disabled
 1: Both HTTP and HTTPS
 2: HTTPS Only
]]
local p_http = webpage_cst_param('Web Interface Protocol',
	webpage_cst_radio('net.http', {'Both HTTP and HTTPS', 'HTTPS Only'},
		{'1', '2'}))

local p_snmp = webpage_cst_param('<abbr title="Simple Network Management Protocol">SNMP</abbr> Support (v1 get only)',
	webpage_cst_enabled('net.srv.snmp'))

local sect_sa
if conf_getdevice('feature.synapps') == '1' then
	local secthd_synapps = webpage_cst_secthd('Syn-Apps', 'sa')

	local hide_sa
	if conf_getpn('synapps.use') == 0 then
		hide_sa = 1
	else
		hide_sa = 0
	end

	local mcast_master = conf_getpn('mcast.mode')
	local disable_sa
	if mcast_master == 1 then
		disable_sa = 1
		hide_sa = 1
	else
		disable_sa = 0
	end

	local sa_disabled_desc = [[SA-Announce Support <br>The SA-Announce feature cannot be used when Multicast Master mode is enabled.  To enable SA-Announce mode, set Multicast Mode to Disabled in "Basic Settings > <a href="/control/shmcast.lua">Multicast</a>"]]

	local secthd_synapps_disabled = webpage_cst_secthd('Syn-Apps', 'sa_disabled',
		 0, sa_disabled_desc)

	local js_synapps = [[
	<script type="text/javascript">
	function display_saserver(mode) {
		var display = (mode == 0) ? 'none' : 'block';
		document.getElementById('synapps_server').style.display =
			display;
		document.getElementById('synapps_port').style.display =
			display;
	}

	function disable_sa(mode) {
		document.getElementById('sa').style.display = mode ? 'none' : 'block';
		document.getElementById('sa_disabled').style.display = mode ? 'block' : 'none';
	}

	disable_sa(]]..disable_sa..[[);

	</script>]]

	local synapps_js0 = [[display_saserver(0);]]
	local synapps_js1 = [[display_saserver(1);]]

	local p_sause = webpage_cst_param('SA-Announce Support',
		webpage_cst_enabled('synapps.use', {synapps_js1, synapps_js0}), nil, 
		'synapps_support', disable_sa)
	local p_saserver = webpage_cst_param('SA-Announce Server',
		webpage_cst_text('synapps.server'),
		'Leave this field blank to use the server provided by DHCP Option 72.',
		'synapps_server', hide_sa)
	local p_saport = webpage_cst_param('Local Management Port',
		webpage_cst_text('synapps.port'), nil, 'synapps_port', hide_sa)

	sect_sa = secthd_synapps_disabled ..
		webpage_subsect_ft .. webpage_sect_ft ..
		secthd_synapps .. webpage_subsect_hd ..
		p_sause .. p_saserver .. p_saport .. js_synapps ..
		webpage_subsect_ft .. webpage_sect_ft
else
	sect_sa = ''
end

local js_valsave = [[<script type="text/javascript">
function validate_save()
{
	if (valPwd(document.getElementById('admin_pwd1'), 'admin_pwd2',
		'inf_admin_pwd', false) == false) {
		alert('Admin Password Mismatch');
		return false;
	} else {
		return true;
	}
}
</script>]]

local form_end_save = '<div class="cbi-page-actions">' ..
	webpage_cst_button('save', 'Save', 'save', 'return validate_save()') ..
	'</div></form><div class="clear"></div>'

mg.write(prod_cst_header() ..
	prod_cst_menu('Advanced Settings', 'Admin') ..
	webpage_cst_mchd(flags['any'], g_reboot, msg) ..
	form_start .. page_start ..
	secthd_pwd .. webpage_subsect_hd ..
	admin_pwd ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_gen .. webpage_subsect_hd ..
	p_dname .. p_welcome .. p_timeout ..
	p_startuptone ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_log .. webpage_subsect_hd .. js_logm ..
	p_log_level .. p_log_method .. p_log_server ..
	webpage_subsect_ft .. webpage_sect_ft ..
--[[	secthd_ha .. webpage_subsect_hd ..
	p_watchdog ..
	webpage_subsect_ft .. webpage_sect_ft .. ]]
	secthd_mgmt .. webpage_subsect_hd ..
	p_http .. p_snmp ..
	webpage_subsect_ft .. webpage_sect_ft ..
	sect_sa ..
	webpage_page_end .. js_valsave .. form_end_save ..
	webpage_mc_ft ..
	prod_footer)
