-- Copyright (c) 2013-2015 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()

local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local flags = {['any'] = 0, ['main'] = 0}
local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)
	conf_update('audio.ring.tone', settings, flags, 'main')
	conf_update('audio.ring.vol', settings, flags, 'main')
	conf_update('phone.timeout.ring', settings, flags, 'main')

	conf_update('audio.page.mode', settings, flags, 'main')
	conf_update('audio.page.vol', settings, flags, 'main')
	conf_update('phone.timeout.inbound', settings, flags, 'main')
	conf_update('audio.page.tone', settings, flags, 'main')
	conf_update('audio.codec.g722', settings, flags, 'main')

	conf_update('audio.noise.use', settings, flags, 'main')
	conf_update('audio.agc.use', settings, flags, 'main')

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['main'] == 1 then
		os.execute('/etc/init.d/algo-main reload')
	end
end
session_save()

local form_start = webpage_cst_formstart('/control/feature.lua')

local feature_start = webpage_cst_pagestart('Features')

local secthd_ring = webpage_cst_secthd('Inbound Ring Settings', 'inbound_ring')

local js_test = [[
<script type="text/javascript">
function test_tone(action)
{
	elems = document.getElementsByName("audio.ring.tone");
	params = 'audio.ring.tone=' + elems[0].value + "&action=" + action;
	xmlhttp.open("POST", "shtonetest.lua", true);
	xmlhttp.setRequestHeader("Content-type",
		"application/x-www-form-urlencoded");
	xmlhttp.send(params);
}

function play_tone()
{
	test_tone("play");
}

function loop_tone()
{
	test_tone("loop");
}

function stop_tone()
{
	test_tone("stop");
}
</script>]]

local p_ring_tone = webpage_cst_param('Ring Tone',
	prod_cst_tonelist('audio.ring.tone') ..
	webpage_cst_fieldbutton('Play', "play_tone();") ..
	webpage_cst_fieldbutton('Loop', "loop_tone();") ..
	webpage_cst_fieldbutton('Stop', "stop_tone();"), nil, 'ring_tone')

local js_apply = [[
<script type="text/javascript">
function adjust_vol(name) {
	elems = document.getElementsByName(name);
	params = name + '=' + elems[0].value;
	xmlhttp.open("POST", "shvol.lua", true);
	xmlhttp.setRequestHeader("Content-type",
		"application/x-www-form-urlencoded");
	xmlhttp.send(params);
}
</script>]]

local ring_enabled = conf_getpn('sip.detect.mode')
local js_show = [[
<script type="text/javascript">
var ring_enabled = ]]..ring_enabled..[[;

function show_page() {
	if (ring_enabled == 0) {
		document.getElementById('inbound_ring').children[0].innerHTML = 'Test Tone Settings';
		document.getElementById('ring_tone').children[0].innerHTML = 'Test Tone';
		document.getElementById('ring_vol').children[0].innerHTML = 'Test Volume';
		document.getElementById('ring_limit').style.display = 'none';
	}
}

show_page();
</script>
]]

local vols = {'10', '9', '8', '7', '6', '5', '4', '3', '2', '1'}
local p_ring_vol = webpage_cst_param('Ring Speaker Volume',
	webpage_cst_select('audio.ring.vol', vols, vols) ..
	webpage_cst_fieldbutton('Apply', "adjust_vol('audio.ring.vol');"), nil, 'ring_vol')

local ring_timeout_desc = {'1 ring', '2 rings', '3 rings', '4 rings',
	'5 rings', '6 rings', 'No limit'}
local ring_timeout_value = {'6', '12', '18', '24', '30', '36', '0'}
local p_ring_timeout = webpage_cst_param('Ring Limit',
	webpage_cst_select('phone.timeout.ring', ring_timeout_desc,
		ring_timeout_value, true),
	'1 ring = 6 seconds.', 'ring_limit')

secthd_page = webpage_cst_secthd('Inbound Page Settings')

local p_page_vol = webpage_cst_param('Page Speaker Volume',
	webpage_cst_select('audio.page.vol', vols, vols) ..
	webpage_cst_fieldbutton('Apply', "adjust_vol('audio.page.vol');"))

local p_page_mode = webpage_cst_param('Page Mode',
	webpage_cst_radio('audio.page.mode', {'One-way', 'Two-way', 'Delayed'},
		{'0', '1', '2'}))

local page_timeout_desc = {'None', '30 seconds', '1 minute', '2 minutes',
	'5 minutes', '10 minutes', '20 minutes', '30 minutes', '60 minutes'}
local page_timeout_vals = {'0', '30', '60', '120',
	'300', '600', '1200', '1800', '3600'}
local p_page_timeout = webpage_cst_param('Page Timeout',
	webpage_cst_select('phone.timeout.inbound',
		page_timeout_desc, page_timeout_vals),
	'Maximum page timeout in Delayed mode is 5 minutes.')

local p_page_tone = webpage_cst_param('Page Tone',
	prod_cst_tonelist('audio.page.tone', {['None']='&lt;None&gt;'}),
	'Use only "page-notif.wav", or custom uploaded file. The other pre-installed tone files all contain silence at the end in order to generate ring "cadence" of 6 seconds. This silence will block the voice path for several seconds at the start of a page.')

local p_g722 = webpage_cst_param('G.722 Support',
	webpage_cst_enabled('audio.codec.g722'))

local secthd_audio = webpage_cst_secthd('Audio Processing')

local p_anc = webpage_cst_param('Ambient Noise Compensation',
	webpage_cst_enabled('audio.noise.use'),
	'Automatically adjust speaker level in response to ambient noise level detected at the device prior to start of each call.')

local p_agc = webpage_cst_param('Automatic Gain Control (AGC)',
	webpage_cst_enabled('audio.agc.use'),
	'Automatically maximize level of voice received from calling phone in order to make page volume more consistent.')

mg.write(prod_cst_header() ..
	prod_cst_menu('Basic Settings', 'Features') ..
	webpage_cst_mchd(flags['any'], 0) ..
	form_start .. feature_start ..
	secthd_ring .. webpage_subsect_hd .. js_test .. js_apply ..
	p_ring_tone .. p_ring_vol .. p_ring_timeout ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_page .. webpage_subsect_hd ..
	p_page_vol .. p_page_mode .. p_page_timeout .. p_page_tone .. p_g722 ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_audio .. webpage_subsect_hd ..
	p_anc .. p_agc ..
	webpage_subsect_ft .. webpage_sect_ft ..
	webpage_page_end .. webpage_form_end_save .. js_show ..
	webpage_mc_ft ..
	webpage_footer)
