-- Copyright (c) 2013-2015 Algo Communication Products Ltd.

require('session')
require('webpage')
require('prod')
require('conf')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local flags = {['any'] = 0, ['reboot'] = 0}
local notif = ''
local vars = {}
local msg = {}

local js_vf = [[
<script>
function validate_fn()
{
	var fwpf="]] .. conf_getdevice('product.firmware.platform') .. [[";
	var fwpd="]] .. conf_getdevice('product.firmware.product') .. [[";

	if (document.getElementsByName('upgopt')[0].checked) {
		var fullfn = document.getElementsByName('firmware')[0].value;
		if (! fullfn || fullfn.length == 0) {
			alert("No firmware file specified.");
			return false;
		}

		var fn = fullfn.substring(fullfn.lastIndexOf('/') + 1);
		if (! fn || fn.length == 0) {
			alert("Not a valid file name.");
			return false;
		}

		var re = new RegExp(fwpd + "-");
		var match = re.exec(fn);
		if (match)
			return true;
		delete re;

		re = new RegExp(fwpf + "-base-");
		match = re.exec(fn);
		if (match)
			return true;
		delete re;

		re = new RegExp(fwpf + "-sys-");
		match = re.exec(fn);
		if (match) {
			alert("Please use URL to upgrade system firmware.");
			return false;
		}
		delete re;

		re = new RegExp(fwpf + "-gui-");
		match = re.exec(fn);
		if (match) {
			alert("Please use URL to upgrade GUI firmware.");
			return false;
		}
		delete re;

		alert('Not a valid firmware file name.');
		return false;
	} else {
		var url = document.getElementsByName('url')[0].value;
		if (! url || url.length == 0) {
			alert("No URL specified.");
			return false;
		}

		var fn = url.substring(url.lastIndexOf('/') + 1);
		if (! fn || fn.length == 0) {
			alert("Not a valid file name.");
			return false;
		}

		var re = new RegExp(fwpd + "-");
		var match = re.exec(fn);
		if (match)
			return true;
		delete re;

		re = new RegExp(fwpf + "-base-");
		match = re.exec(fn);
		if (match)
			return true;
		delete re;

		re = new RegExp(fwpf + "-sys-");
		match = re.exec(fn);
		if (match)
			return true;
		delete re;

		re = new RegExp(fwpf + "-gui-");
		match = re.exec(fn);
		if (match)
			return true;
		delete re;

		alert('Not a valid firmware file name.');
		return false;
	}
}
</script>]]

local js_downloading = [[<script>
var dl_interval_id = 0;

function dl_check_state()
{
	xmlhttp.open("GET", "/control/downloading.lua",true);
	xmlhttp.onreadystatechange=function() {
		if (xmlhttp.readyState==4) {
			if (xmlhttp.status==200) {
				var resp = xmlhttp.responseText;

				if (resp == "downloading") {
					return;
				} else if (resp == "finished") {
					window.location = "/control/upgrading.lua";
				} else {
					dlmsg.className="errorbox";
					dltitle.innerHTML="Download Failed.";
					dldesc.innerHTML=resp;
				}
				clearInterval(interval_id);
			}
		}
	}
	xmlhttp.send(null)
}

window.onload = function () {
	clearInterval(dl_interval_id);
	dl_interval_id = setInterval("dl_check_state()", 1000);
}
</script>]]

function check_cmdrc(cmd)
	local state, cause, rc = os.execute(cmd)
	return rc
end

conf_setp('upgopt', '0')
conf_setp('url', '')

local cl = mg.request_info.http_headers['Content-Length']
if cl then
	os.execute('rm -rf /tmp/web/mp/*')
	webpage_parse_multipart(cl, vars)

	if vars['restore'] then
		local conf_fn = vars['settings']['filename']
		local msgtitle = 'Restore Failed'
		if conf_fn == '' then
			msg[msgtitle] = 'No file chosen.'
		elseif vars['settings']['filesize'] == 0 then
			msg[msgtitle] = 'Empty file.'
		elseif vars['settings']['filesize'] > 1048576 then
			msg[msgtitle] = 'File too large (> 1048576 bytes).'
		else
			os.execute('/opt/algo/bin/restore-settings "'
				.. conf_fn .. '"')
			conf_load()
			os.execute('/opt/algo/bin/apply-settings all')
			flags['any'] = 1
			flags['reboot'] = 1
			g_reboot = 1
			session_save()
		end
	elseif vars['reset'] then
		os.execute('/opt/algo/bin/restore-default')
		conf_load()
		os.execute('/opt/algo/bin/apply-settings all')
		flags['any'] = 1
		flags['reboot'] = 1
		g_reboot = 1
		session_save()
	elseif vars['upgrade'] then
		local msgtitle = 'Upgrade Failed'
		local fw_fn, md5_fn
		if vars['upgopt'] == '0' then
			fw_fn = vars['firmware']['filename']
			md5_fn = vars['md5']['filename']
		else
			fw_fn = vars['url']:sub(vars['url']:find('[^/]*$'))
			md5_fn = fw_fn:sub(1, -3) .. 'md5'
		end

		local fwpf = string.gsub(
			conf_getdevice('product.firmware.platform'), '-', '(-)')
		local fwpd = string.gsub(
			conf_getdevice('product.firmware.product'), '-', '(-)')

		if g_reboot == 1 then
			msg[msgtitle] = 'Please finish rebooting before upgrade.'
		elseif string.find(fw_fn, fwpf) == nil and
			string.find(fw_fn, fwpd) == nil then
			msg[msgtitle] = 'Wrong firmware image file.'
		elseif fw_fn == '' then
			msg[msgtitle] = 'No firmware file specified.'
		elseif md5_fn == '' then
			msg[msgtitle] = 'No MD5 file specified.'
		elseif vars['upgopt'] == '0' then
			if vars['firmware']['filesize'] == 0 then
				msg[msgtitle] = 'Empty firmware file.'
			elseif vars['md5']['filesize'] == 0 then
				msg[msgtitle] = 'Empty MD5 file.'
			elseif 0 ~= check_cmdrc('/opt/algo/bin/check-firmware '
					.. '/tmp/web/mp' .. ' ' .. md5_fn) then
				print('check-firmware rc=', os.execute('/opt/algo/bin/check-firmware ' .. '/tmp/web/mp' .. ' ' .. md5_fn))
				msg[msgtitle] = 'Invalid firmware or MD5.'
			elseif 0 ~= check_cmdrc('/opt/algo/bin/check-sysreq '
					.. '/tmp/web/mp ' .. fw_fn .. ' '
					.. '/var/run/sysreq.status') then
				msg[msgtitle] = webpage_read_file(
					'/var/run/sysreq.status', '')
			else
				os.execute('/opt/algo/bin/upgrade-firmware ' ..
					'-w /tmp/web/mp ' .. fw_fn .. ' &')
				session_redirect('/control/upgrading.lua')
				do return end
			end
		elseif  vars['upgopt'] == '1' then
			os.execute('rm -f /var/run/download.status')
			os.execute('/opt/algo/bin/download-firmware ' ..
				vars['url'] .. ' &')
			notif = '<div id="dlmsg" class="loadbox"><strong><div id="dltitle">Downloading Firmware</div></strong><br /><div id="dldesc">Please do not disconnect the device during firmware download or switch to another page.</div></div>' .. js_downloading
		end

		conf_setp('upgopt', vars['upgopt'])
		conf_setp('url', vars['url'])
	end
end
session_save()

local form_start = [[
<form method="post" enctype="multipart/form-data" action="/control/maint.lua">]]

local page_start = webpage_cst_pagestart('System Maintenance')

local secthd_bk = webpage_cst_secthd('Backup / Restore Configuration')

local prod = conf_getdevice('product.name')
local mac = webpage_read_file('/sys/class/net/eth0/address')
mac=string.gsub(mac:upper(), '%W', '')
local js_bkconf = string.format(
	[[window.location.href='/control/settings-%s-%s.txt']],
	prod, mac)
local p_backup = webpage_cst_param('Download Configuration File',
	webpage_cst_button('backup', 'Download', 'download', js_bkconf, 1))
local p_restore = webpage_cst_param('Restore Configuration File',
	form_start ..
	webpage_cst_upload('settings') ..
	webpage_cst_button('restore', 'Restore', 'upload') ..
	webpage_form_end)

local js_reset = [[return confirm('All the settings will be restored to factory default values.  Do you want to continue?');]]
local p_reset = webpage_cst_param('Restore Configuration to Defaults',
	form_start ..
	webpage_cst_button('reset', 'Restore Defaults', 'reset', js_reset) ..
	webpage_form_end)

local secthd_reboot = webpage_cst_secthd('Reboot')

local js_reboot = [[return confirm('Do you want to reboot the unit?');]]
local p_reboot = webpage_cst_param('Reboot the device',
	webpage_cst_formstart('/control/reboot.lua') ..
	webpage_cst_button('reboot', 'Reboot', 'reset', js_reboot) ..
	webpage_form_end)

local secthd_upgrade = webpage_cst_secthd('Upgrade to New Firmware')

local form_upgrade = [[
<form id="upg_form" method="post" enctype="multipart/form-data" action="/control/maint.lua" onsubmit="return validate_fn();">]]

local js_upg = [[
<script type="text/javascript">
function display_upg_options(mode) {
	if (mode == 0) {
		document.getElementById("upg_fw").style.display = 'block';
		document.getElementById("upg_md5").style.display = 'block';
		document.getElementById("upg_url").style.display = 'none';
	} else {
		document.getElementById("upg_fw").style.display = 'none';
		document.getElementById("upg_md5").style.display = 'none';
		document.getElementsByName("firmware")[0].value = '';
		document.getElementsByName("md5")[0].value = '';
		document.getElementById("upg_url").style.display = 'block';
	}
}
</script>]]

if conf_getp('upgopt') == '' then
	conf_setp('upgopt', '0')
end
local hide = conf_getpn('upgopt')
local upg_js0 = [[display_upg_options(0);]]
local upg_js1 = [[display_upg_options(1);]]
local p_upgopt = webpage_cst_param('Method',
	webpage_cst_radio('upgopt', {'From Local Files', 'From URL'},
		{'0', '1'}, {upg_js0, upg_js1}))

local p_fw = webpage_cst_param('Firmware Image', webpage_cst_upload('firmware'),
	nil, 'upg_fw', hide)
local p_md5 = webpage_cst_param('MD5 Checksum', webpage_cst_upload('md5'),
	nil, 'upg_md5', hide)
local p_url = webpage_cst_param('Upgrade URL', webpage_cst_text('url'),
	nil, 'upg_url', 1 - hide)
local p_upgrade = webpage_cst_param('',
	webpage_cst_button('upgrade', 'Upgrade', 'upload'))

-- prevent these paramaters to be saved into config file
conf_remove('upgopt')
conf_remove('url')

mg.write(prod_cst_header() ..
	prod_cst_menu('System', 'Maintenance') ..
	webpage_cst_mchd(flags['any'], flags['reboot'], msg) .. notif ..
	page_start ..
	secthd_bk .. webpage_subsect_hd ..
	p_backup .. p_restore .. p_reset .. webpage_subsect_ft ..
	webpage_sect_ft ..
	secthd_reboot .. webpage_subsect_hd ..
	p_reboot ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_upgrade .. webpage_subsect_hd .. js_upg ..
	js_vf .. form_upgrade .. p_upgopt .. p_fw .. p_md5 .. p_url ..
	p_upgrade .. webpage_form_end ..
	webpage_subsect_ft .. webpage_sect_ft ..
	webpage_page_end .. webpage_mc_ft ..
	prod_footer)
