-- Copyright (c) 2013-2015 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local flags = {['any'] = 0, ['reboot'] = 0}

local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)
	conf_update('prov.use', settings, flags, 'reboot')
	conf_update('prov.server.method', settings, flags, 'reboot')
	conf_update('prov.server.static', settings, flags, 'reboot')
	conf_update('prov.download.method', settings, flags, 'reboot')
	conf_update('prov.auth.user', settings, flags, 'reboot')
	conf_update('prov.auth.pwd', settings, flags, 'reboot')
	conf_update('prov.download.cfgpath', settings, flags, 'reboot')
	conf_update('prov.download.fwpath', settings, flags, 'reboot')

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['reboot'] == 1 then
		g_reboot = 1
		session_save()
	end
end
session_save()

local form_start = webpage_cst_formstart('/control/prov.lua')

local page_start = webpage_cst_pagestart('Provisioning Settings')

local secthd_mode = webpage_cst_secthd('Mode')

local js_use = [[
<script type="text/javascript">
function display_use(mode) {
	var display = (mode == 0) ? 'none' : 'block';
	document.getElementById("prov_settings").style.display = display;
}
</script>]]
local use_js0 = [[display_use(0);]]
local use_js1 = [[display_use(1);]]
local p_use = webpage_cst_param('Provisioning Mode',
	webpage_cst_enabled('prov.use', {use_js1, use_js0}))

local secthd_settings = webpage_cst_secthd('Settings', 'prov_settings',
	1 - conf_getpn('prov.use'))

local sm_value = {'option66', 'static'}
local sm_desc = {'DHCP Option 66', 'Static'}
local js_static = [[
<script type="text/javascript">
function display_static(mode) {
	var display = (mode == 0) ? 'none' : 'block';
	document.getElementById("server_static").style.display = display;
}
</script>]]
local static_js0 = [[display_static(0);]]
local static_js1 = [[display_static(1);]]
local p_server_method = webpage_cst_param('Server Method',
	webpage_cst_radio('prov.server.method', sm_desc, sm_value,
		{static_js0, static_js1}),
	nil, 'server_method')

local static_hide
if conf_getp('prov.server.method') ~= 'option66' then
	static_hide = 0
else
	static_hide = 1
end
local p_static_server = webpage_cst_param('Static Server',
	webpage_cst_text('prov.server.static'), nil, 'server_static',
	static_hide)

local js_dlm = [[
<script type="text/javascript">
function display_dlauth(mode) {
	var display = (mode == 0) ? 'none' : 'block';
	document.getElementById("auth_user").style.display = display;
	document.getElementById("auth_pwd").style.display = display;
}
</script>]]
local dlm_js0 = [[display_dlauth(0);]]
local dlm_js1 = [[display_dlauth(1);]]

local dlm_value = {'tftp', 'ftp', 'http'}
local dlm_desc = {'TFTP', 'FTP', 'HTTP'}
local p_download_method = webpage_cst_param('Download Method',
	webpage_cst_radio('prov.download.method', dlm_desc, dlm_value,
		{dlm_js0, dlm_js1, dlm_js1}),
	nil, 'download_method')

local auth_hide
if conf_getp('prov.download.method') ~= 'tftp' then
	auth_hide = 0
else
	auth_hide = 1
end
local p_auth_user = webpage_cst_param('Auth User Name',
	webpage_cst_text('prov.auth.user'), nil, 'auth_user', auth_hide)

local p_auth_pwd = webpage_cst_pwd('Auth Password',
	'prov.auth.pwd', 'auth_pwd', auth_hide)

local p_cfgpath = webpage_cst_param('Config Download Path',
	webpage_cst_text('prov.download.cfgpath'), nil, 'cfgpath')

local p_fwpath = webpage_cst_param('Firmware Download Path',
	webpage_cst_text('prov.download.fwpath'), nil, 'fwpath')

mg.write(prod_cst_header() ..
	prod_cst_menu('Advanced Settings', 'Provisioning') ..
	webpage_cst_mchd(flags['any'], flags['reboot']) ..
	form_start .. page_start ..
	secthd_mode .. webpage_subsect_hd .. js_use ..
	p_use ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_settings .. webpage_subsect_hd .. js_static ..
	p_server_method .. p_static_server ..
	webpage_subsect_ft .. webpage_subsect_hd .. js_dlm ..
	p_download_method .. p_auth_user .. p_auth_pwd ..
	webpage_subsect_ft .. webpage_subsect_hd ..
	p_cfgpath .. p_fwpath ..
	webpage_subsect_ft .. webpage_sect_ft ..
	webpage_page_end .. webpage_form_end_save ..
	webpage_mc_ft ..
	prod_footer)
