-- Copyright (c) 2013-2015 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end
session_save()

local flags = {['any'] = 0, ['main'] = 0}
local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)
	conf_update('audio.drc.use', settings, flags, 'main')
	conf_update('audio.drc.gain', settings, flags, 'main')
	conf_update('audio.jc.range', settings, flags, 'main')

	if conf_getdevice('product.name') == '8301' then
		conf_update('audio.dtmf.use', settings, flags, 'main')
	end

	if conf_getdevice('product.name') ~= '8039' then
		conf_update('audio.filter.g711spk', settings, flags, 'main')
		conf_update('audio.filter.g711mic', settings, flags, 'main')
		conf_update('audio.filter.g722spk', settings, flags, 'main')
		conf_update('audio.filter.g722mic', settings, flags, 'main')
		conf_update('audio.filter.noisespk', settings, flags, 'main')
		conf_update('audio.filter.noisemic', settings, flags, 'main')
	end

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['main'] == 1 then
		os.execute('/etc/init.d/algo-main reload')
	end
end

local form_start = webpage_cst_formstart('/control/shadvaud.lua')

local page_start = webpage_cst_pagestart('Advanced Audio Functions')

local secthd_adv = webpage_cst_secthd('Functions')

local js_drc = [[<script type="text/javascript">
function display_drc(mode)
{
	var display = (mode == 0) ? 'none' : 'block';
	document.getElementById("drc_gain").style.display = display;
}
</script>]]

local drc_js0 = [[display_drc(0);]]
local drc_js1 = [[display_drc(1);]]
local hide = 1 - conf_getpn('audio.drc.use')
local p_drc = webpage_cst_param('Dynamic Range Compression (DRC)',
	webpage_cst_enabled('audio.drc.use', {drc_js1, drc_js0}),
	'Compress the dynamic range of page audio to increase loudness.')

local drc_value = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'}
local p_drc_gain = webpage_cst_param('Dynamic Range Compression Gain',
	webpage_cst_select('audio.drc.gain', drc_value, drc_value),
	'Specify the amount of compression gain.  More gain increases distortion.',
	'drc_gain', hide)

local p_range = webpage_cst_param('Jitter Buffer Range (milliseconds, 10 ~ 500)',
	webpage_cst_text('audio.jc.range'),
	'Adds more buffering if necessary to correct for inconsistent delays on the network. Use of the lowest value generally is recommended.')

local p_dtmf
if conf_getdevice('product.name') == '8301' then
	p_dtmf = webpage_cst_param('Generate In-Band DTMF Tones',
		webpage_cst_enabled('audio.dtmf.use'),
		'Play DTMF tones during a SIP Call to allow interoperability with DTMF-controlled multi-zone amplifiers')
else
	p_dtmf = ''
end

local fullsect_filter
if conf_getdevice('product.name') == '8039' then
	fullsect_filter = ''
else
	local secthd_filter = webpage_cst_secthd('Audio Filters')

	local filter_desc = {'150Hz High-Pass', '300Hz High-Pass',
		'500Hz High-Pass', 'None'}
	local filter_value = {'0', '1', '2', '3'}

	local p_filter_g711spk = webpage_cst_param('Speaker Filter G.711',
		webpage_cst_select('audio.filter.g711spk', filter_desc,
			filter_value),
		'Bandwidth also limited by audio codecs.')

	local p_filter_g711mic = webpage_cst_param('Microphone Filter G.711',
		webpage_cst_select('audio.filter.g711mic', filter_desc,
			filter_value))

	local p_filter_g722spk = webpage_cst_param('Speaker Filter G.722',
		webpage_cst_select('audio.filter.g722spk', filter_desc,
			filter_value))

	local p_filter_g722mic = webpage_cst_param('Microphone Filter G.722',
		webpage_cst_select('audio.filter.g722mic', filter_desc,
			filter_value))
	--[[
	local p_ns = webpage_cst_param('Noise Suppression',
		webpage_cst_enabled('audio.ns.use'),
		'Use active noise suppression to reduce noise.  May introduce audible artefacts.')
	--]]
	local p_spk_nse = webpage_cst_param('Speaker Noise Filter',
		webpage_cst_enabled('audio.filter.noisespk'),
		'Heavy filtering below 150Hz to reduce mains induced noise (fans).')

	local p_mic_nse = webpage_cst_param('Microphone Noise Filter',
		webpage_cst_enabled('audio.filter.noisemic'),
		'Heavy filtering below 150Hz to reduce mains induced noise (fans).')
	fullsect_filter = secthd_filter .. webpage_subsect_hd ..
		p_filter_g711spk .. p_filter_g711mic ..
		p_filter_g722spk .. p_filter_g722mic ..
		p_spk_nse .. p_mic_nse ..
		webpage_subsect_ft .. webpage_sect_ft
end

mg.write(prod_cst_header() ..
	prod_cst_menu('Advanced Settings', 'Advanced Audio') ..
	webpage_cst_mchd(flags['any'], 0) ..
	form_start .. page_start ..
	secthd_adv .. webpage_subsect_hd .. js_drc ..
	p_drc .. p_drc_gain .. p_range .. p_dtmf .. 
	webpage_subsect_ft .. webpage_sect_ft ..
	fullsect_filter ..
	webpage_page_end .. webpage_form_end_save ..
	webpage_mc_ft ..
	prod_footer)
