-- Copyright (c) 2013-2016 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local nzones = 50
local flags = {['any'] = 0, ['main'] = 0}

local cl = mg.request_info.http_headers['Content-Length']
if cl then

	local settings = {}
	webpage_parse_post(cl, settings)

	local zonename
	local tonename
	local slavetonename
	local volname
	for i = 1, nzones do
		zonename = 'mcast.zone' .. i
		tonename = 'mcast.tone' .. i
		slavetonename = 'mcast.slavetone' .. i
		volname = 'mcast.vol' .. i
		conf_update(zonename, settings, flags, 'main')
		conf_update(tonename, settings, flags, 'main')
		conf_update(slavetonename, settings, flags, 'main')
		conf_update(volname, settings, flags, 'main')
	end

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['main'] == 1 then
		os.execute('/etc/init.d/algo-main reload')
	end
end
session_save()

local mcast_mode = conf_getp('mcast.mode')
local exp_mode = conf_getp('mcast.zones.exp')

local form_start = webpage_cst_formstart('/control/shadvmcast.lua')

local page_start

local mcast_mode_name
if mcast_mode == '0' then
	mcast_mode_name = 'Disabled'
elseif mcast_mode == '1' then
	mcast_mode_name = 'Master'
else
	mcast_mode_name = 'Slave'
end

if mcast_mode == '0' then
	page_start = webpage_cst_pagestart('Advanced Multicast Settings')
else
	-- Link to 8373-specific multicast page
	if conf_getdevice('product.name') == '8373' then
		page_start = webpage_cst_pagestart('Advanced Multicast Settings',
			'<img src="/static/information-white.png"> Current multicast mode: ' ..
			mcast_mode_name ..
			'<br>Multicast mode can be set in \"Basic Settings > <a href="/control/multicast.lua">Multicast</a>\"')
	else
		page_start = webpage_cst_pagestart('Advanced Multicast Settings',
			'<img src="/static/information-white.png"> Current multicast mode: ' ..
			mcast_mode_name ..
			'<br>Multicast mode can be set in \"Basic Settings > <a href="/control/shmcast.lua">Multicast</a>\"')
	end
end

local js_show = [[
<script type="text/javascript">

function show_class(name, state)
{
	var elements = document.getElementsByClassName(name);
	for (var i = 0; i < elements.length; i++) {
		elements[i].style.display = state ? 'inline' : 'none';
	}
}

function show_element(id, state)
{
	document.getElementById(id).style.display = state ? 'block' : 'none';
}

function apply_control(mcast_mode, expanded_mode)
{
	if (mcast_mode == 0) {
		show_element('mcast_disabled', 1);
		show_element('zone_sect', 0);
		show_element('exp_zone_sect', 0);
	} else if (mcast_mode == 1) {			// Multicast master
		show_element('mcast_disabled', 0);
		show_element('zone_sect', 1);
		show_element('exp_zone_sect', expanded_mode);
		show_element('column_vol', 0);
		show_element('column_vol_exp', 0);

		show_class('zone_tone', 1);
		show_class('zone_slavetone', 0);
		show_class('zone_vol', 0);
	} else if (mcast_mode == 2) {	// Multicast slave
		show_element('mcast_disabled', 0);
		show_element('zone_sect', 1);
		show_element('exp_zone_sect', expanded_mode);

		show_class('zone_tone', 0);
		show_class('zone_slavetone', 1);
		show_class('zone_vol', 1);
	}
}

function show_page()
{
	var expanded_mode = ]] .. exp_mode .. [[;
	var mcast_mode = ]] .. mcast_mode .. [[;
	apply_control(mcast_mode, expanded_mode);
}
</script>]]
local mode_action = 'show_page();'

local mcast_disabled_desc = [[To edit multicast zone definitions, enable multicast in "Basic Settings > <a href='/control/shmcast.lua'>Multicast</a>"]]

local p_mcast_disabled = webpage_cst_secthd('Multicast Disabled', 'mcast_disabled',
		 0, mcast_disabled_desc)

local slave_tone_desc
if mcast_mode == "2" then
	slave_tone_desc = '<img src="/static/information-white.png"> When an Algo device is the multicast master, a page tone will play on the slave device, so it is recommended to set the slave tone to "None".'
else
	slave_tone_desc = ''
end

local secthd_zone = webpage_cst_secthd('Basic Zone Definition',
	'zone_sect', 1, slave_tone_desc)

local secthd_exp_zone = webpage_cst_secthd('Expanded Zone Definition',
	'exp_zone_sect', 1)

local volume_val = {'10', '9', '8', '7', '6', '5', '4', '3', '2', '1', '0'}
local volume_desc = {'Volume 10', 'Volume 9', 'Volume 8', 'Volume 7',
	'Volume 6', 'Volume 5', 'Volume 4', 'Volume 3', 'Volume 2', 'Volume 1',
	'&ltUse Default Page Volume&gt'}
local tone_extras = {}
tone_extras['None'] = '&ltNone&gt'
tone_extras['Default'] = '&ltUse Default Page Tone&gt'

local zone_vol_class = 'zone_vol'
local zone_tone_class = 'zone_tone'
local zone_slavetone_class = 'zone_slavetone'

function generate_vol_sel(zone)
	local vol_conf = 'mcast.vol' .. zone
	return string.format(webpage_cst_select(vol_conf,
		 volume_desc, volume_val, nil, nil, zone_vol_class))
end

local zone_tone_desc = {}
local zone_tone_value = {}
webpage_cst_tonelist_values(zone_tone_desc, zone_tone_value, tone_extras)

function cst_zone(desc, zone)
	return webpage_cst_param_lesswhitespace(desc,
		webpage_cst_text('mcast.zone' .. zone) .. '<label> </label>' ..
		webpage_cst_select('mcast.tone' .. zone,
			zone_tone_desc, zone_tone_value,
			nil, nil, zone_tone_class) ..
		webpage_cst_select('mcast.slavetone' .. zone,
			zone_tone_desc, zone_tone_value, nil, nil, zone_slavetone_class) ..
		'<label> </label>' ..
		webpage_cst_select('mcast.vol' .. zone, volume_desc, volume_val,
			nil, nil, zone_vol_class))
end

local p_columns = webpage_cst_param_lesswhitespace('<b style="font-size:11pt;">Zone</b>',
	webpage_cst_columnheader('<b>IP Address and Port</b>') .. '<label> </label>' ..
	webpage_cst_columnheader('<b>Page Tone</b>') .. '<label> </label>' ..
	webpage_cst_columnheader('<b>Page Volume</b>', 'column_vol'))

local p_zone_p = cst_zone('Priority Call (DTMF:9)', 9)
local p_zone_a = cst_zone('All Call (DTMF:0)', 8)
local p_zone1 = cst_zone('Zone 1 (DTMF:1)', 1)
local p_zone2 = cst_zone('Zone 2 (DTMF:2)', 2)
local p_zone3 = cst_zone('Zone 3 (DTMF:3)', 3)
local p_zone4 = cst_zone('Zone 4 (DTMF:4)', 4)
local p_zone5 = cst_zone('Zone 5 (DTMF:5)', 5)
local p_zone6 = cst_zone('Zone 6 (DTMF:6)', 6)
local p_zone_m = cst_zone('Music (DTMF:7)', 7)

local p_exp_zone
local p_exp_zones_txt = ''

for i = 10, nzones do
	p_exp_zone = webpage_cst_param_lesswhitespace('Zone ' .. i .. ' (DTMF: *' .. i .. ')',
		webpage_cst_text('mcast.zone' .. i) .. '<label> </label>' ..
		webpage_cst_select('mcast.tone' .. i,
			zone_tone_desc, zone_tone_value,
			nil, nil, zone_tone_class) ..
		webpage_cst_select('mcast.slavetone' .. i,
			zone_tone_desc, zone_tone_value,
			nil, nil, zone_slavetone_class) ..
		'<label> </label>' ..
		generate_vol_sel(i))

	p_exp_zones_txt = p_exp_zones_txt .. p_exp_zone
end

local p_columns_exp = webpage_cst_param_lesswhitespace('<b style="font-size:11pt;">Zone</b>',
	webpage_cst_columnheader('<b>IP Address and Port</b>') .. '<label> </label>' ..
	webpage_cst_columnheader('<b>Page Tone</b>') .. '<label> </label>' ..
	webpage_cst_columnheader('<b>Page Volume</b>', 'column_vol_exp'))

p_exp_zones_txt = p_columns_exp .. p_exp_zones_txt

local js_showall = [[<script type="text/javascript">
show_page();
</script>]]
-- Make this page a fixed width so that the select boxes don't wrap
mg.write(prod_cst_header(nil, nil, nil, nil, nil, nil, nil, 1) ..
	prod_cst_menu('Advanced Settings', 'Advanced Multicast') ..
	webpage_cst_mchd(flags['any'], 0) ..
	form_start .. page_start ..
	js_show ..
	secthd_zone .. webpage_subsect_hd ..
	p_columns ..
	p_zone_p .. p_zone_a .. p_zone1 .. p_zone2 .. p_zone3 .. p_zone4 ..
	p_zone5 .. p_zone6 .. p_zone_m ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_exp_zone .. webpage_subsect_hd ..
	p_exp_zones_txt ..
	webpage_subsect_ft .. webpage_sect_ft ..
	p_mcast_disabled ..
	webpage_subsect_ft .. webpage_sect_ft ..
	js_showall ..
	webpage_page_end .. webpage_form_end_save ..
	webpage_mc_ft ..
	prod_footer)

