-- Copyright (c) 2014-2015 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local flags = {['any']=0, ['main']=0}
local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)
	conf_update('audio.port.input', settings, flags, 'main')
	conf_update('audio.streaming.mode', settings, flags, 'main')

	conf_update('io.relayin.mode', settings, flags, 'main')

	conf_update('relayin.trigger.action',  settings, flags, 'main')
	conf_update('relayin.trigger.dest',  settings, flags, 'main')
	conf_update('relayin.trigger.tone',  settings, flags, 'main')
	conf_update('relayin.trigger.duration',  settings, flags, 'main')
	conf_update('relayin.trigger.interval',  settings, flags, 'main')
	conf_update('relayin.trigger.maxtone',  settings, flags, 'main')
	conf_update('relayin.trigger.restart', settings, flags, 'main')

	conf_update('relayin.tamper.action',  settings, flags, 'main')
	conf_update('relayin.tamper.dest',  settings, flags, 'main')
	conf_update('relayin.tamper.tone',  settings, flags, 'main')
	conf_update('relayin.tamper.duration',  settings, flags, 'main')
	conf_update('relayin.tamper.interval',  settings, flags, 'main')
	conf_update('relayin.tamper.maxtone',  settings, flags, 'main')

	conf_update('callbox.call.action',  settings, flags, 'main')
	conf_update('callbox.call.dest',  settings, flags, 'main')
	conf_update('callbox.call.tone',  settings, flags, 'main')
	conf_update('callbox.call.duration',  settings, flags, 'main')
	conf_update('callbox.call.interval',  settings, flags, 'main')
	conf_update('callbox.call.maxtone',  settings, flags, 'main')
	conf_update('callbox.cancel.action',  settings, flags, 'main')

	conf_update('phone.timeout.ringback', settings, flags, 'main')
	conf_update('phone.ringback.use', settings, flags, 'main')
	conf_update('phone.timeout.outbound', settings, flags, 'main')

	conf_update('sd.loud.use', settings, flags, 'main')
	conf_update('sd.loud.sens', settings, flags, 'main')
	conf_update('sd.loud.dest', settings, flags, 'main')

	conf_update('io.led.heartbeat', settings, flags, 'main')

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['main'] == 1 then
		os.execute('/etc/init.d/algo-main reload')
	end
end
session_save()

local form_start = webpage_cst_formstart('/control/shio.lua')

local io_start = webpage_cst_pagestart('Input/Output')

local secthd_general = webpage_cst_secthd('General', 'sect_general')

local secthd_disabled = webpage_cst_secthd('Relay Input Mode Disabled', 'sect_disabled')

local secthd_stream = webpage_cst_secthd('Audio Streaming', 'sect_stream')

local show_audio_input

if conf_getdevice('product.name') == '8301' then
	show_audio_input = [[
		// Show or hide the audio input selection
		var hide_tone = 0;
		document.getElementById('audio_in_port').style.display = 'block';
		if (mode == 1 || mode == 2 || mode == 3 || mode == 4) {
			if (trigger_action == 1 || trigger_action == 3) {
				hide_tone = 1;
			}
			if (trigger_action == 3) {
				document.getElementById('trigger_restart').style.display =
					hide_tone ? 'none' : 'block';
				document.getElementById('trigger_dest').style.display =
					hide_tone ? 'none' : 'block';
			}
		}

		document.getElementById('trigger_tone').style.display =
			hide_tone ? 'none' : 'block';
		document.getElementById('trigger_maxtone').style.display =
			hide_tone ? 'none' : 'block';
		document.getElementById('trigger_interval').style.display =
			hide_tone ? 'none' : 'block';
		document.getElementById('audio_in_port').style.display =
			hide_tone ? 'block' : 'none';

		// Don't allow audio streaming always on if relay trigged audio streaming enabled
		document.getElementById('sect_stream').style.display =
			hide_tone ? 'none' : 'block';
		document.getElementById('audio_alwayson_disabled').style.display =
			hide_tone ? 'block' : 'none';
	]]
else
	show_audio_input = ''
end

local js_action = [[
<script type="text/javascript">

function mirror_opt_alwayson()
{
	var sel;
	var index = document.getElementsByName('audio.port.streaminput')[0].selectedIndex;
	sel = document.getElementsByName('audio.port.input')[0];
	sel.selectedIndex = index;
}

function mirror_opt_relay()
{
	var sel;
	var index = document.getElementsByName('audio.port.input')[0].selectedIndex;
	sel = document.getElementsByName('audio.port.streaminput')[0];
	sel.selectedIndex = index;
}

function control_mode(mode)
{
	var hide_trigger, hide_tamper;

	if (mode == 0 || mode == 5) {
		hide_trigger = 1;
		hide_tamper = 1;
		hide_callbox = 1;
	} else if (mode == 1 || mode == 3) {
		hide_trigger = 0;
		hide_tamper = 1;
		hide_callbox = 1;
	} else if (mode == 2 || mode == 4) {
		hide_trigger = 0;
		hide_tamper = 0;
		hide_callbox = 1;
	} else {	// mode 6 callbox
		hide_trigger = 1;
		hide_tamper = 1;
		hide_callbox = 0;
	}

	document.getElementById('sect_trigger').style.display =
		hide_trigger ? 'none' : 'block';
	document.getElementById('sect_tamper').style.display =
		hide_tamper ? 'none' : 'block';
	document.getElementById('sect_callbox').style.display =
		hide_callbox ? 'none' : 'block';
}

function apply_control(sect, action, duration)
{
	var fname;
	var hide_dest, hide_restart, hide_tone, hide_duration;
	var hide_interval, hide_maxtone;

	if (sect == 'trigger' || sect == 'tamper') {
		fname = 'relayin.' + sect + '.duration';
	} else {	// sect == 'callbox'
		fname = 'callbox.call.duration';
	}

	if (action == 0) {
		hide_dest = 1;
		hide_restart = 1;
		hide_tone = 0;
		hide_duration = 0;
		if (duration == 0) {
			hide_interval = 1;
			hide_maxtone = 1;
		} else {
			hide_interval = 0;
			hide_maxtone = 0;
		}
	} else if (action == 1) {
		hide_dest = 0;
		hide_restart = 0;
		hide_tone = 1;
		hide_duration = 1;
		hide_interval = 1;
		hide_maxtone = 1;
	} else {
		hide_dest = 0;
		hide_restart = 0;
		hide_tone = 0;
		hide_duration = 1;
		hide_interval = 0;
		hide_maxtone = 0;
	}

	document.getElementById(sect + '_dest').style.display =
		hide_dest ? 'none' : 'block';
	if (sect == 'trigger') {
		document.getElementById(sect + '_restart').style.display =
			hide_restart ? 'none' : 'block';
	}
	document.getElementById(sect + '_tone').style.display =
		hide_tone ? 'none' : 'block';
	document.getElementById(sect + '_duration').style.display =
		hide_duration ? 'none' : 'block';
	document.getElementById(sect + '_interval').style.display =
		hide_interval ? 'none' : 'block';
	document.getElementById(sect + '_maxtone').style.display =
		hide_maxtone ? 'none' : 'block';

}

function read_radio_value(radios)
{
	var val = 0;

	for (var i = 0, length = radios.length; i < length; i++) {
		if (radios[i].checked) {
			val = parseInt(radios[i].value);
			break;
		}
	}
	return val;
}

function hide_all(enabled)
{
	document.getElementById('sect_disabled').style.display = enabled ? 'block' : 'none';
	document.getElementById('sect_general').style.display = enabled ? 'none' : 'block';
	document.getElementById('sect_trigger').style.display = enabled ? 'none' : 'block';
	document.getElementById('sect_tamper').style.display = enabled ? 'none' : 'block';
	document.getElementById('sect_callbox').style.display = enabled ? 'none' : 'block';
	document.getElementById('outbound_sip').style.display = enabled ? 'none' : 'block';
}

function show_page(io_disabled)
{
	var mode;
	var duration;
	var trigger_action;
	var tamper_action;
	var callbox_action;
	var hide_outbound_sip = 1;

	if (io_disabled) {
		hide_all(1);
	} else {
		hide_all(0);
		document.getElementById('sect_disabled').style.display = 'none';
		mode = read_radio_value(
			document.getElementsByName('io.relayin.mode'));
		control_mode(mode);

		trigger_action = read_radio_value(
			document.getElementsByName('relayin.trigger.action'));
		duration = read_radio_value(
			document.getElementsByName('relayin.trigger.duration'));
		apply_control('trigger', trigger_action, duration);

		tamper_action = read_radio_value(
			document.getElementsByName('relayin.tamper.action'));
		duration = read_radio_value(
			document.getElementsByName('relayin.tamper.duration'));
		apply_control('tamper', tamper_action, duration);

		callbox_action = read_radio_value(
			document.getElementsByName('callbox.call.action'));
		duration = read_radio_value(
			document.getElementsByName('callbox.call.duration'));
		apply_control('callbox', callbox_action, duration);

		// Show or hide outbound SIP section
		if ((mode == 1 || mode == 3) &&
				(trigger_action == 1 || trigger_action == 2)) {
			hide_outbound_sip = 0;
		}

		if ((mode == 2 || mode == 4) &&
				(trigger_action == 1 || tamper_action == 1 ||
				 trigger_action == 2 || tamper_action == 2 )) {
			hide_outbound_sip = 0;
		}

		if (mode == 6 &&
				(callbox_action == 1 || callbox_action == 2)) {
			hide_outbound_sip = 0;
		}
		
		document.getElementById('outbound_sip').style.display =
			hide_outbound_sip ? 'none' : 'block';

]] .. show_audio_input .. [[

	}
}
</script>]]


local mode_action = 'show_page();'

local p_relayin_mode = webpage_cst_param('Relay Input Mode',
	webpage_cst_radio('io.relayin.mode',
		{'Disabled',
		 'Relay Normally Open',
		 'Relay Normally Open with Supervision (e.g. Algo 1203 Call Switch)',
		 'Relay Normally Closed',
		 'Relay Normally Closed with Supervision',
		 'Speaker Mute',
		 'Call Button'},
		{'0', '1', '2', '3', '4', '5', '6'},
		{mode_action, mode_action, mode_action, mode_action,
		 mode_action, mode_action, mode_action},
		nil, nil, nil, 1))

local audio_alwayson_desc = [["Audio Always On" will play sound on local speaker as well as multicast if configured.]]

local p_audio_alwayson = webpage_cst_param('Audio Always On',
	webpage_cst_radio('audio.streaming.mode',
		{'Enabled', 'Disabled'}, {'1', '0'},
		{mode_action, mode_action}), audio_alwayson_desc)

local alwayson_disabled_desc = [[Audio Streaming Always On cannot be used when the relay trigger action is set to stream audio.  To enable Audio Streaming Always On, set Relay Input Mode to "Disabled", or set Action When Input Triggered to "Play Tone" or "Make SIP Call with Tone".]]

local p_audio_alwayson_disabled = webpage_cst_secthd('Audio Streaming', 'audio_alwayson_disabled',
		 0, alwayson_disabled_desc)

local disabled_desc = [[The Relay Input feature cannot be used when the Audio Input Port is set to a relay-controlled mode. To enable the Relay Input features, set the Audio Input Port to "None" in the Basic Settings > <a href="/control/feature.lua">Features</a> tab. <br>Note that the Audio Input Port can also be used if Broadcast Mode is set to "Always active multicast master" (as this doesn't require the relay), in which case the Relay Input feature will have higher priority and can pre-empt the audio stream from the input port.]]

local p_relayin_disabled = webpage_cst_secthd('', 'relayin_disabled',
	0, disabled_desc)

local secthd_trigger = webpage_cst_secthd('Action When Input Triggered',
	'sect_trigger')

local action_label = 'Action'

local trigger_action_opts
local trigger_action_vals

local action_opts
local action_vals

local action_desc
if conf_getdevice('product.name') == '8301' then
	action_desc = [["Play Tone" and "Stream Audio" will play sound on local speaker as well as multicast if configured.]]
else
	action_desc = [["Play Tone" will play sound on local speaker as well as multicast if configured.]]
end

if conf_getdevice('product.name') == '8301' then
	trigger_action_opts = {'Play Tone', 'Make SIP Voice Call',
		'Make SIP Call with Tone', 'Stream Audio'}
	trigger_action_vals = {'0', '1', '2', '3'}
else
	trigger_action_opts = {'Play Tone', 'Make SIP Voice Call',
		'Make SIP Call with Tone'}
	trigger_action_vals = {'0', '1', '2'}
end

action_opts = {'Play Tone', 'Make SIP Voice Call',
	'Make SIP Call with Tone'}
action_vals = {'0', '1', '2'}

local p_trigger_action = webpage_cst_param(action_label,
	webpage_cst_radio('relayin.trigger.action',
		trigger_action_opts, trigger_action_vals,
		{mode_action, mode_action, mode_action, mode_action}),
	action_desc)

local dest_label = 'Dialing Extension'
local dest_desc = [[SIP account required in Page Extension fields in order to make a call.]]
local p_trigger_dest = webpage_cst_param(dest_label,
	webpage_cst_text('relayin.trigger.dest'), dest_desc,
	'trigger_dest')

local restart_label = 'Allow 2nd Button Press to End & Restart Call'
local p_trigger_restart = webpage_cst_param(restart_label,
	webpage_cst_enabled('relayin.trigger.restart'),
	nil, 'trigger_restart')

local audio_in_desc = {'Aux In (front blue 3.5mm headset)',
	'Aux Out (front green 3.5mm headset with mic)',
	'Line In (back terminal block & XLR)'}
local audio_in_vals = {'1', '2', '3'}

local p_audio_in
if conf_getdevice('product.name') == '8301' then
	p_audio_in = webpage_cst_param('Audio Input Port',
		webpage_cst_select('audio.port.input',
			audio_in_desc, audio_in_vals), nil, 'audio_in_port', nil)
else
	p_audio_in = ''
end

local p_audio_in_const = webpage_cst_param('Audio Input Port',
	webpage_cst_select('audio.port.streaminput',
		audio_in_desc, audio_in_vals))

local tone_label = 'Tone/Pre-recorded Announcement'
local p_trigger_tone = webpage_cst_param(tone_label,
	webpage_cst_tonelist('relayin.trigger.tone'),
	nil, 'trigger_tone')

local duration_label = 'Tone Duration'
local duration_opts = {'Play Once', 'Play While Held'}
local duration_vals = {'0', '1'}
local p_trigger_duration = webpage_cst_param(duration_label,
	webpage_cst_radio('relayin.trigger.duration',
		duration_opts, duration_vals,
		{mode_action, mode_action}),
	nil, 'trigger_duration')

local interval_label = 'Interval Between Tones (seconds)'
local p_trigger_interval = webpage_cst_param(interval_label,
	webpage_cst_text('relayin.trigger.interval'), nil,
	'trigger_interval')

local maxtone_label = 'Maximum Tone Duration'
local maxdur_desc = {'None', '1 minute', '2 minutes', '3 minutes', '5 minutes',
	'10 minutes', '15 minutes', '20 minutes', '30 minutes', '45 minutes',
	'60 minutes'}
local maxdur_value = {'0', '60', '120', '180', '300', '600',
	'900', '1200', '1800', '2700', '3600'}
local p_trigger_maxtone = webpage_cst_param(maxtone_label,
	webpage_cst_select('relayin.trigger.maxtone',
		maxdur_desc, maxdur_value), nil,
	'trigger_maxtone')

local secthd_tamper = webpage_cst_secthd('Action When Tamper Detected',
	'sect_tamper')

local p_tamper_action = webpage_cst_param(action_label,
	webpage_cst_radio('relayin.tamper.action',
		action_opts, action_vals,
		{mode_action, mode_action, mode_action}),
	action_desc)

local p_tamper_dest = webpage_cst_param(dest_label,
	webpage_cst_text('relayin.tamper.dest'), dest_desc,
	'tamper_dest')

local p_tamper_tone = webpage_cst_param(tone_label,
	webpage_cst_tonelist('relayin.tamper.tone'),
	nil, 'tamper_tone')

local p_tamper_duration = webpage_cst_param(duration_label,
	webpage_cst_radio('relayin.tamper.duration',
		duration_opts, duration_vals,
		{mode_action, mode_action}),
	nil, 'tamper_duration')

local p_tamper_interval = webpage_cst_param(interval_label,
	webpage_cst_text('relayin.tamper.interval'), nil,
	'tamper_interval')

local p_tamper_maxtone = webpage_cst_param(maxtone_label,
	webpage_cst_select('relayin.tamper.maxtone',
		maxdur_desc, maxdur_value), nil,
	'tamper_maxtone')

local secthd_callbox = webpage_cst_secthd('Call Button Settings',
	'sect_callbox')

local p_callbox_action = webpage_cst_param(action_label,
	webpage_cst_radio('callbox.call.action',
		action_opts, action_vals,
		{mode_action, mode_action, mode_action}),
	action_desc)

local p_callbox_dest = webpage_cst_param(dest_label,
	webpage_cst_text('callbox.call.dest'), dest_desc,
	'callbox_dest')

local p_callbox_tone = webpage_cst_param(tone_label,
	webpage_cst_tonelist('callbox.call.tone'),
	nil, 'callbox_tone')

duration_opts = {'Play Once', 'Play Until Cancelled'}
duration_vals = {'0', '2'}
local p_callbox_duration = webpage_cst_param(duration_label,
	webpage_cst_radio('callbox.call.duration',
		duration_opts, duration_vals,
		{mode_action, mode_action}),
	nil, 'callbox_duration')

local p_callbox_interval = webpage_cst_param(interval_label,
	webpage_cst_text('callbox.call.interval'), nil,
	'callbox_interval')

local p_callbox_maxtone = webpage_cst_param(maxtone_label,
	webpage_cst_select('callbox.call.maxtone',
		maxdur_desc, maxdur_value), nil,
	'callbox_maxtone')

local p_callbox_cancel = webpage_cst_param(
	'Allow Cancel Button to End Tone/Call',
	webpage_cst_enabled('callbox.cancel.action'), nil,
	'callbox_cancel')

local secthd_sip = webpage_cst_secthd('Outbound SIP Call Settings', 'outbound_sip')

local ringback_desc = {'1 ring', '2 rings', '3 rings',
	'4 rings', '5 rings', '6 rings', '7 rings',
	'8 rings', '9 rings', 'No limit'}
local ringback_value = {'6', '12', '18', '24', '30', '36', '42', '48', '54',
	'0'}
local ringback_timeout_desc = [[1 ring = 6 seconds]]
local p_ringback_timeout = webpage_cst_param('Outbound Ring Limit',
	webpage_cst_select('phone.timeout.ringback',
		ringback_desc, ringback_value),
	ringback_timeout_desc)

local p_ringback_tone = webpage_cst_param('Ringback Tone',
	webpage_cst_enabled('phone.ringback.use'))

local p_maxcall = webpage_cst_param('Maximum Call Duration',
	webpage_cst_select('phone.timeout.outbound',
		maxdur_desc, maxdur_value))

--[[
local secthd_loud = webpage_cst_secthd('Loud Sound Detection',
	'sect_loud')

local js_loud = '<script type="text/javascript">
function control_loud(inuse)
{
	document.getElementById('loud_sens').style.display =
		inuse ? 'block' : 'none';
	document.getElementById('loud_dest').style.display =
		inuse ? 'block' : 'none';
}
</script>'

local p_loud_use = webpage_cst_param('Detection Function',
	webpage_cst_enabled('sd.loud.use',
		{'control_loud(1);', 'control_loud(0);'}))

local hide_loud = 1 - conf_getpn('sd.loud.use')

local p_loud_sens = webpage_cst_param('Sensitivity',
	webpage_cst_text('sd.loud.sens'), nil,
	'loud_sens', hide_loud)

local p_loud_dest = webpage_cst_param(dest_label,
	webpage_cst_text('sd.loud.dest'), dest_desc,
	'loud_dest', hide_loud)
]]

local secthd_output = webpage_cst_secthd('Output')

local p_heartbeat = webpage_cst_param('Heartbeat Light',
	webpage_cst_enabled('io.led.heartbeat'),
	'Flash the blue light every 30 seconds to indicate that the speaker is powered and running.')

local disable_relayin = 0
local audio_port = conf_getpn('audio.port.input')

if conf_getdevice('product.name') == '8301' then
	if audio_port ~= 0 then
		disable_relayin = 1
	end
end

local js_showall = [[<script type="text/javascript">
show_page(0);

mirror_opt_relay();
var inputsel1 = document.getElementsByName('audio.port.streaminput')[0];
var inputsel2 = document.getElementsByName('audio.port.input')[0];

inputsel1.addEventListener("change", mirror_opt_alwayson);
inputsel2.addEventListener("change", mirror_opt_relay);
</script>]]

mg.write(prod_cst_header() ..
	prod_cst_menu('Additional Features', 'Input/Output') ..
	webpage_cst_mchd(flags['any'], 0) ..
	form_start .. io_start ..
	secthd_general .. webpage_subsect_hd .. js_action ..
	p_relayin_mode ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_disabled ..
	p_relayin_disabled ..
	webpage_subsect_ft ..
	webpage_sect_ft)

if conf_getdevice('product.name') == '8301' then
	mg.write(secthd_stream .. webpage_subsect_hd ..
	p_audio_alwayson .. p_audio_in_const ..
	webpage_subsect_ft .. webpage_sect_ft ..
	p_audio_alwayson_disabled ..
	webpage_subsect_ft .. webpage_sect_ft)
end

	mg.write(secthd_trigger .. webpage_subsect_hd ..
	p_trigger_action .. p_trigger_dest .. p_trigger_restart ..
	p_audio_in ..
	p_trigger_tone ..
	p_trigger_duration .. p_trigger_interval .. p_trigger_maxtone ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_tamper .. webpage_subsect_hd ..
	p_tamper_action .. p_tamper_dest .. p_tamper_tone ..
	p_tamper_duration .. p_tamper_interval .. p_tamper_maxtone ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_callbox .. webpage_subsect_hd ..
	p_callbox_action .. p_callbox_dest .. p_callbox_tone ..
	p_callbox_duration .. p_callbox_interval .. p_callbox_maxtone ..
	p_callbox_cancel ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_sip .. webpage_subsect_hd ..
	p_ringback_timeout .. p_ringback_tone .. p_maxcall ..
	webpage_subsect_ft .. webpage_sect_ft
--[[	secthd_loud .. webpage_subsect_hd .. js_loud ..
	p_loud_use .. p_loud_sens .. p_loud_dest ..
	webpage_subsect_ft .. webpage_sect_ft .. ]]
	)
	
if conf_getdevice('product.name') ~= '8301' then
	mg.write(secthd_output .. webpage_subsect_hd .. p_heartbeat ..
		webpage_subsect_ft .. webpage_sect_ft)
end

mg.write(webpage_page_end .. webpage_form_end_save .. js_showall ..
	webpage_mc_ft ..
	prod_footer)
