-- Copyright (c) 2013-2015 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local nzones = 50
local flags = {['any'] = 0, ['main'] = 0}

local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	local szone_list = {}
	for i = 1, nzones do
		szone_list[i] = 'szone' .. i
	end

	local mzone_list = {}
	for i = 1, nzones do
		mzone_list[i] = 'mzone' .. i
	end

	webpage_parse_post(cl, settings)
	conf_update('mcast.mode', settings, flags, 'main')
	conf_update('mcast.zones.exp', settings, flags, 'main')
	conf_update('mcast.zones.select', settings, flags, 'main')
	conf_update('mcast.master.codec',  settings, flags, 'main')
	conf_update('mcast.master.ptime',  settings, flags, 'main')
	conf_update('audio.jc.delay', settings, flags, 'main')
	conf_update('mcast.master.fixed', settings, flags, 'main')

	local val = ''
	for index, value in ipairs(szone_list) do
		if settings[value] ~= nil then
			val = val .. tostring(index) .. ','
		end
	end

	settings['mcast.slave.zones'] = val
	conf_update('mcast.slave.zones', settings, flags, 'main')

	val = ''	
	for index, value in ipairs(mzone_list) do
		if settings[value] ~= nil then
			val = val .. tostring(index) .. ','
		end
	end

	settings['mcast.master.zones'] = val
	conf_update('mcast.master.zones', settings, flags, 'main')

	conf_update('mcast.polycom.mode', settings, flags, 'main')
	conf_update('mcast.polycom.zone', settings, flags, 'main')
	conf_update('mcast.polycom.default', settings, flags, 'main')
	conf_update('mcast.polycom.prio', settings, flags, 'main')
	conf_update('mcast.polycom.emerg', settings, flags, 'main')

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['main'] == 1 then
		os.execute('/etc/init.d/algo-main reload')
	end
end
session_save()

local form_start = webpage_cst_formstart('/control/shmcast.lua')

local page_start = webpage_cst_pagestart('Multicast Settings')

local secthd_mode = webpage_cst_secthd('Multicast Mode', 1)

local js_show = [[
<script type="text/javascript">
// global variables
var nzones = 50;

function show_exp_zone_opts(state)
{
	var zonesel = document.getElementsByName('mcast.master.fixed')[0];
	var val = zonesel.options[zonesel.selectedIndex].value;
	var len = 0;

	zonesel.options.length = 0;
	zonesel.options[0] = new Option('Priority Call', 9);
	zonesel.options[1] = new Option('All Call', 8);
	zonesel.options[2] = new Option('Music', 7);
	for (var i = 1; i <= 6; i++) {
		zonesel.options[zonesel.options.length] =
			new Option('Zone ' + i, i);
	}

	if (state) {
		for (var i = 10; i <= nzones; i++) {
			zonesel.options[zonesel.options.length] =
				new Option('Zone *' + i, i);
		}
	}

	var found = 0;
	for (var i = 0; i < zonesel.options.length; i++) {
		if (val == zonesel.options[i].value) {
			zonesel.selectedIndex = i;
			found = 1;
			break;
		}
	}
	if (! found) {
		zonesel.options[zonesel.options.length] =
			new Option(val + ' (?)', val);
		zonesel.selectedIndex = zonesel.options.length - 1;
	}
}

function show_element(id, state)
{
	document.getElementById(id).style.display = state ? 'block' : 'none';
}

function show_class(name, state)
{
	var elements = document.getElementsByClassName(name);
	for (var i = 0; i < elements.length; i++) {
		elements[i].style.display = state ? 'inline' : 'none';
	}
}

function disable_mcast_dual(state)
{
	var elements = document.getElementsByClassName('mcast_dual');
	for (var i = 0; i < elements.length; i++) {
		elements[i].style.color = state ? '#888' : '#000';
	}
	document.getElementsByName('mcast.polycom.mode')[3].disabled = state;
	document.getElementsByName('mcast.polycom.mode')[4].disabled = state;
}

function apply_control(mcast_mode, polycom_mode, expanded_mode, zonesel_mode)
{
	if (mcast_mode == 0) {
		show_element('master_params', 0);
		show_element('slave_params', 0);
		show_element('master_sect', 0);
		show_element('slave_sect', 0);
		show_element('polycom_sect', 0);
		show_element('num_zone_sel', 0);
		show_element('master_exp_zones', 0);
	} else if (mcast_mode == 1) {	// mcast master
		show_element('master_params', 1);
		show_element('slave_params', 0);
		show_element('slave_sect', 0);
		show_element('polycom_sect', 1);
		show_class('zone_tone', 1);
		show_class('zone_vol', 0);
		disable_mcast_dual(0);
		show_class('mcast_dual', 1);
		show_element('polycom_params', polycom_mode);
		if (polycom_mode == 0 ||
			polycom_mode == 3 ||
			polycom_mode == 4) {
			show_element('num_zone_sel', 1);
			show_element('master_sect', 1);
		} else {
			show_element('num_zone_sel', 0);
			show_element('master_sect', 0);
		}
		show_element('polycom_prio', 0);
		show_element('polycom_emerg', 0);
		if (expanded_mode == 0) {
			show_element('master_exp_zones', 0);
			show_exp_zone_opts(0);
		} else {
			show_element('master_exp_zones', 1);
			show_exp_zone_opts(1);
		}
		show_element('fixed_zone', 1);
	} else {	// mcast slave
		show_element('master_params', 0);
		show_element('slave_params', 1);
		show_element('master_sect', 0);
		show_element('polycom_sect', 1);
		show_element('master_exp_zones', 0);
		show_class('zone_tone', 0);
		show_class('zone_vol', 1);
		if (polycom_mode == 3 || polycom_mode == 4) {
			disable_mcast_dual(1);
		} else {
			show_class('mcast_dual', 0);
		}
		if (polycom_mode == 0) {
			show_element('num_zone_sel', 1);
			show_element('slave_sect', 1);
			show_element('polycom_params', 0);
			show_element('slave_exp_zones', expanded_mode);
			show_element('polycom_prio', 0);
			show_element('polycom_emerg', 0);
		} else {
			show_element('num_zone_sel', 0);
			show_element('slave_sect', 0);
			show_element('polycom_params', 1);
			show_element('slave_exp_zones', 0);
			show_element('polycom_prio', 1);
			show_element('polycom_emerg', 1);
		}
	}
}

function read_radio_value(radios)
{
	var val = 0;

	for (var i = 0, length = radios.length; i < length; i++) {
		if (radios[i].checked) {
			val = parseInt(radios[i].value);
			break;
		}
	}
	return val;
}

function show_page()
{
	var mcast_mode = read_radio_value(
		document.getElementsByName('mcast.mode'));

	var polycom_mode = read_radio_value(
		document.getElementsByName('mcast.polycom.mode'));

	var expanded_mode = read_radio_value(
		document.getElementsByName('mcast.zones.exp'));

	var zonesel_mode = read_radio_value(
		document.getElementsByName('mcast.zones.select'));

	apply_control(mcast_mode, polycom_mode, expanded_mode, zonesel_mode);
}
</script>]]
local mode_action = 'show_page();'

local p_mode = webpage_cst_param('Multicast Mode',
	webpage_cst_radio('mcast.mode',
		{'None', 'Master/Sender', 'Slave/Receiver'}, {"0", "1", "2"},
		{mode_action, mode_action, mode_action}, true), 'Multicast Zone Definitions can be found in \"Advanced Settings > <a href="/control/shadvmcast.lua">Advanced Multicast</a>\".')

local master_params_hd = webpage_cst_subsect_hd('master_params', 1)

local p_codec = webpage_cst_param('Master Output Codec',
	webpage_cst_select('mcast.master.codec',
		{'G.711 ulaw', 'G.722'}, {'1', '2'}))

local p_ptime = webpage_cst_param(
	'Master Output Packetization Time (milliseconds)',
	webpage_cst_select('mcast.master.ptime',
		{'20', '30', '40'}, {'20', '30', '40'}))

local slave_params_hd = webpage_cst_subsect_hd('slave_params', 1)

local p_delay = webpage_cst_param('Audio Delay (milliseconds, 0 ~ 1000)',
	webpage_cst_text('audio.jc.delay'),
	'Use this feature to synchronize audio output if using multicast with other third-party devices that have a delay in their audio path.  Applies to Multicast Slave mode only.')

local secthd_master = webpage_cst_secthd("Master/Sender Zone Settings",
	'master_sect', 1)

local p_select = webpage_cst_param('Zone Selection Mode',
	webpage_cst_radio('mcast.zones.select',
		{'DTMF Selectable Zone', 'Single Zone'}, {'1', '0'}))

local zone_list = {
	'Zone 1  ', 'Zone 2  ', 'Zone 3  ', 'Zone 4  ', 'Zone 5  ', 'Zone 6  ', 
	'Music   ', 'All Call', 'Priority Call', 
	'Zone *10', 
	'Zone *11', 'Zone *12', 'Zone *13', 'Zone *14', 'Zone *15',
	'Zone *16', 'Zone *17', 'Zone *18', 'Zone *19', 'Zone *20',
	'Zone *21', 'Zone *22', 'Zone *23', 'Zone *24', 'Zone *25',
	'Zone *26', 'Zone *27', 'Zone *28', 'Zone *29', 'Zone *30',
	'Zone *31', 'Zone *32', 'Zone *33', 'Zone *34', 'Zone *35',
	'Zone *36', 'Zone *37', 'Zone *38', 'Zone *39', 'Zone *40',
	'Zone *41', 'Zone *42', 'Zone *43', 'Zone *44', 'Zone *45',
	'Zone *46', 'Zone *47', 'Zone *48', 'Zone *49', 'Zone *50'
	}

local fixed_list = {zone_list[9], zone_list[8], zone_list[1], zone_list[2],
	zone_list[3], zone_list[4], zone_list[5], zone_list[6], zone_list[7]}
local fixed_value = {'9', '8', '1', '2', '3', '4', '5', '6', '7'}

local num_zones_desc = {'Basic Zones Only', 'Basic and Expanded Zones'}
local num_zones_vals = {'0', '1'} 

local p_num_zones = webpage_cst_param('Number of Zones',
	webpage_cst_radio('mcast.zones.exp', num_zones_desc, num_zones_vals,
		{mode_action, mode_action}, true),
		nil, 'num_zone_sel', 1)

local singlezone_desc

if conf_getdevice('product.name') == '8301' then
	singlezone_desc = 'If "DTMF Selectable Zone" is selected above, then this single zone setting will not apply to Paging (since the zone can now be dynamically selected per call using DTMF), but it will still apply to Relay triggered events, including the analog audio input.'
else
	singlezone_desc = 'If "DTMF Selectable Zone" is selected above, then this single zone setting will not apply to Paging (since the zone can now be dynamically selected per call using DTMF), but it will still apply to Relay triggered events.'
end

local p_fixed = webpage_cst_param('Master Single Zone',
	webpage_cst_select('mcast.master.fixed', fixed_list,
		fixed_value), singlezone_desc, 'fixed_zone')

local js_zonesel = [[
<script type="text/javascript">
function checkbox_toggle_all(name, checked) {
	var i;
	var fullname;
	for (i = 1; i <= nzones; i++) {
		fullname = name+i;
		document.getElementsByName(fullname)[0].checked = checked;
	}
}
</script>]]

--[[
local p_ring = webpage_cst_param('Master Ring Mode',
	webpage_cst_radio('mcast.master.ring',
		{'Send Control Signal', 'Send Audio Stream'}))
]]

local secthd_slave = webpage_cst_secthd("Slave/Receiver Zone Settings",
	'slave_sect', 1)

local zone_checked = {}
for i = 1, nzones do
	zone_checked[i] = ''
end

local mzone_checked = {}
for i = 1, nzones do
	mzone_checked[i] = ''
end

local mcast_szone_val = conf_getp('mcast.slave.zones')
for c in string.gmatch(mcast_szone_val, '([^,]+)') do
	local n = tonumber(c)
	if n ~= nil and n >= 1 and n <= nzones then
		zone_checked[n] = ' checked'
	end
end

local mcast_mzone_val = conf_getp('mcast.master.zones')
for c in string.gmatch(mcast_mzone_val, '([^,]+)') do
	local n = tonumber(c)
	if n ~= nil and n>= 1 and n <= nzones then
		mzone_checked[n] = ' checked'
	end
end

local zonefmt = '<input type="checkbox" name="szone%d" value="1"%s>%s'
local mzonefmt = '<input type="checkbox" name="mzone%d" value="1"%s>%s'
function cst_checkbox_szone3(z1, z2, z3)
	return string.format(zonefmt, z1, zone_checked[z1], zone_list[z1]) ..
		string.format(zonefmt, z2, zone_checked[z2], zone_list[z2]) ..
		string.format(zonefmt, z3, zone_checked[z3], zone_list[z3])
end

function cst_checkbox_szone5(z1, z2, z3, z4, z5)
	return string.format(zonefmt, z1, zone_checked[z1], zone_list[z1]) ..
		string.format(zonefmt, z2, zone_checked[z2], zone_list[z2]) ..
		string.format(zonefmt, z3, zone_checked[z3], zone_list[z3]) ..
		string.format(zonefmt, z4, zone_checked[z4], zone_list[z4]) ..
		string.format(zonefmt, z5, zone_checked[z5], zone_list[z5])
end

function cst_checkbox_mzone3(z1, z2, z3)
	return string.format(mzonefmt, z1, mzone_checked[z1], zone_list[z1]) ..
		string.format(mzonefmt, z2, mzone_checked[z2], zone_list[z2]) ..
		string.format(mzonefmt, z3, mzone_checked[z3], zone_list[z3])
end

function cst_checkbox_mzone5(z1, z2, z3, z4, z5)
	return string.format(mzonefmt, z1, mzone_checked[z1], zone_list[z1]) ..
		string.format(mzonefmt, z2, mzone_checked[z2], zone_list[z2]) ..
		string.format(mzonefmt, z3, mzone_checked[z3], zone_list[z3]) ..
		string.format(mzonefmt, z4, mzone_checked[z4], zone_list[z4]) ..
		string.format(mzonefmt, z5, mzone_checked[z5], zone_list[z5])
end

local p_szones = webpage_cst_param('Basic Slave Zones',
	cst_checkbox_szone3(9, 8, 7) .. '<br />' ..
	cst_checkbox_szone3(1, 2, 3) .. '<br />' ..
	cst_checkbox_szone3(4, 5, 6), nil, nil, 0)

local p_expzones = webpage_cst_param('Expanded Slave Zones',
	cst_checkbox_szone5(10, 11, 12, 13, 14) .. '<br />' ..
	cst_checkbox_szone5(15, 16, 17, 18, 19) .. '<br />' ..
	cst_checkbox_szone5(20, 21, 22, 23, 24) .. '<br />' ..
	cst_checkbox_szone5(25, 26, 27, 28, 29) .. '<br />' ..
	cst_checkbox_szone5(30, 31, 32, 33, 34) .. '<br />' ..
	cst_checkbox_szone5(35, 36, 37, 38, 39) .. '<br />' ..
	cst_checkbox_szone5(40, 41, 42, 43, 44) .. '<br />' ..
	cst_checkbox_szone5(45, 46, 47, 48, 49) .. '<br />' ..
	string.format(zonefmt, 50, zone_checked[50], zone_list[50]) ..
	'<br />' ..
	webpage_cst_fieldbutton('Select All',
		"checkbox_toggle_all('szone', 1)") ..
	webpage_cst_fieldbutton('Clear All', "checkbox_toggle_all('szone', 0)"),
	nil, 'slave_exp_zones')

local p_master_zones = webpage_cst_param('Speaker Playback Zones',
		cst_checkbox_mzone3(9, 8, 7) .. '<br />' ..
		cst_checkbox_mzone3(1, 2, 3) .. '<br />' ..
		cst_checkbox_mzone3(4, 5, 6),
		'Allows master device to play audio for selected zones only. This is useful if using DTMF Selectable Zone mode (or <a href="/control/shext.lua">More Page Extensions</a> per zone) and wishing to make the Master unit a member of only certain zones.',
		nil, 0)

local p_master_exp_zones = webpage_cst_param('Expanded Speaker Playback Zones',
		cst_checkbox_mzone5(10, 11, 12, 13, 14) .. '<br />' ..
		cst_checkbox_mzone5(15, 16, 17, 18, 19) .. '<br />' ..
		cst_checkbox_mzone5(20, 21, 22, 23, 24) .. '<br />' ..
		cst_checkbox_mzone5(25, 26, 27, 28, 29) .. '<br />' ..
		cst_checkbox_mzone5(30, 31, 32, 33, 34) .. '<br />' ..
		cst_checkbox_mzone5(35, 36, 37, 38, 39) .. '<br />' ..
		cst_checkbox_mzone5(40, 41, 42, 43, 44) .. '<br />' ..
		cst_checkbox_mzone5(45, 46, 47, 48, 49) .. '<br />' ..
		string.format(mzonefmt, 50, mzone_checked[50], zone_list[50]) ..
		'<br />' ..
		webpage_cst_fieldbutton('Select All',
			"checkbox_toggle_all('mzone', 1)") ..
		webpage_cst_fieldbutton('Clear All', "checkbox_toggle_all('mzone', 0)"),
		nil, 'master_exp_zones')

local secthd_polycom = webpage_cst_secthd('Polycom Group Paging/Push-to-Talk',
	'polycom_sect', 1)

local mcast_dual = 'mcast_dual'
local mcast_reg = 'mcast_regular'

local p_polycom_mode = webpage_cst_param('Multicast Type',
	webpage_cst_radio('mcast.polycom.mode',
		{'Regular (RTP)',
		 'Polycom Group Page',
		 'Polycom Push-to-Talk',
		 'Regular RTP + Polycom Group Page',
		 'Regular RTP + Polycom Push-to-Talk'}, 
		{'0', '1', '2', '3', '4'},
		{mode_action, mode_action, mode_action, mode_action,
			mode_action},
		true,
		nil,
		{mcast_reg, mcast_reg, mcast_reg, mcast_dual, mcast_dual}, 1),
		[[Regular mode uses RTP audio packets compatible with all Algo SIP endpoints, and most multicast-enabled phones.]])

local group_list = {}
local group_value = {}
for i = 1, 25 do
	group_list[i] = 'Group ' .. i
	group_value[i] = tostring(i)
end

if conf_getp('mcast.polycom.default') == '' then
	conf_update('mcast.polycom.default', '1')
end
if conf_getp('mcast.polycom.prio') == '' then
	conf_update('mcast.polycom.prio', '24')
end
if conf_getp('mcast.polycom.emerg') == '' then
	conf_update('mcast.polycom.emerg', '25')
end

local polycom_hd = webpage_cst_subsect_hd('polycom_params', 1)
local p_polycom_zone = webpage_cst_param('Polycom Zone',
	webpage_cst_text('mcast.polycom.zone'),
	'Enter the same Multicast IP Address & Port number as configured on the Polycom phones.')
local p_polycom_default = webpage_cst_param('Polycom Default Channel',
	webpage_cst_select('mcast.polycom.default', group_list, group_value),
	nil, 'polycom_default')
local p_polycom_prio = webpage_cst_param('Polycom Priority Channel',
	webpage_cst_select('mcast.polycom.prio', group_list, group_value),
	nil, 'polycom_prio')
local p_polycom_emerg = webpage_cst_param('Polycom Emergency Channel',
	webpage_cst_select('mcast.polycom.emerg', group_list, group_value),
	nil, 'polycom_emerg')

local js_showall = [[<script type="text/javascript">
show_page();
</script>]]

mg.write(prod_cst_header() ..
	prod_cst_menu('Basic Settings', 'Multicast') ..
	webpage_cst_mchd(flags['any'], 0) ..
	form_start .. page_start ..
	secthd_mode .. webpage_subsect_hd .. 
	js_show ..
	p_mode .. p_num_zones ..
	webpage_subsect_ft ..
	master_params_hd ..
	p_codec .. p_ptime ..
	webpage_subsect_ft .. slave_params_hd ..
	p_delay ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_polycom .. webpage_subsect_hd ..
	p_polycom_mode ..
	webpage_subsect_ft .. polycom_hd ..
	p_polycom_zone .. p_polycom_default ..
	p_polycom_prio .. p_polycom_emerg ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_master .. webpage_subsect_hd ..
	p_select .. p_fixed .. 
	p_master_zones .. p_master_exp_zones ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_slave .. webpage_subsect_hd ..
	js_zonesel ..
	p_szones .. p_expzones ..
	webpage_subsect_ft .. webpage_sect_ft ..
	webpage_page_end .. webpage_form_end_save .. js_showall ..
	webpage_mc_ft ..
	prod_footer)
