-- Copyright (c) 2013-2015 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local flags = {['any']=0, ['main']=0}
local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)
	conf_update('sip.proxy', settings, flags, 'main')
	conf_update('sip.detect.mode', settings, flags, 'main')
	conf_update('sip.page.use', settings, flags, 'main')
	conf_update('sip.alert1.user', settings, flags, 'main')
	conf_update('sip.alert1.auth', settings, flags, 'main')
	conf_update('sip.alert1.pwd', settings, flags, 'main')
	conf_update('sip.alert1.event', settings, flags, 'main')
	conf_update('sip.u1.user', settings, flags, 'main')
	conf_update('sip.u1.auth', settings, flags, 'main')
	conf_update('sip.u1.pwd', settings, flags, 'main')

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['main'] == 1 then
		os.execute('/etc/init.d/algo-main reload')
	end
end
session_save()

local form_start = webpage_cst_formstart('/control/shsip.lua')

local page_start = webpage_cst_pagestart('SIP Settings')

local secthd_basic = webpage_cst_secthd("SIP")

local proxy_label = [[
<abbr title="Session Initiation Protocol">SIP</abbr> Domain (Proxy Server)]]

local proxy_desc = [[
Default port is 5060. To specify a different port, enter PROXY:PORT, e.g. my_proxy.com:5070, or 192.168.1.10:5080.]]

local p_proxy = webpage_cst_param(proxy_label,
	webpage_cst_text('sip.proxy'), proxy_desc)

local js_control = [[
<script type="text/javascript">
function display_element(id, state)
{
	document.getElementById(id).style.display = state ? 'block' : 'none';
}

function control_display(alert_mode, page_inuse)
{
	if (alert_mode == 0) {
		display_element('alert_ext', 0);
		display_element('alert_auth', 0);
		display_element('alert_pwd', 0);
		display_element('alert_event', 0);
		document.getElementById("alert_desc").innerHTML="Ring Extension";
	} else if (alert_mode == 1) {
		display_element('alert_ext', 1);
		display_element('alert_auth', 1);
		display_element('alert_pwd', 1);
		display_element('alert_event', 0);
		document.getElementById("alert_desc").innerHTML="Ring Extension";
	} else if (alert_mode == 2) {
		display_element('alert_ext', 1);
		display_element('alert_auth', 0);
		display_element('alert_pwd', 0);
		display_element('alert_event', 1);
		document.getElementById("alert_desc").innerHTML="Ring/Alert Extension";
	}

	if (page_inuse == 0) {
		if (alert_mode == 2) {
			display_element('sip_ext', 1);
			display_element('sip_auth', 1);
			display_element('sip_pwd', 1);
		} else {
			display_element('sip_ext', 0);
			display_element('sip_auth', 0);
			display_element('sip_pwd', 0);
		}
		document.getElementById("base_desc").innerHTML="SIP Extension";
	} else {
		display_element('sip_ext', 1);
		display_element('sip_auth', 1);
		display_element('sip_pwd', 1);
		document.getElementById("base_desc").innerHTML="Page Extension";
	}
}

function control_alert(alert_mode)
{
	var page_inuse = 0;
	var radios = document.getElementsByName('sip.page.use');
	for (var i = 0, length = radios.length; i < length; i++) {
		if (radios[i].checked) {
			page_inuse = parseInt(radios[i].value);
			break;
		}
	}
	control_display(alert_mode, page_inuse);
}

function control_page(page_inuse)
{
	var alert_mode = 0;
	var radios = document.getElementsByName('sip.detect.mode');
	for (var i = 0, length = radios.length; i < length; i++) {
		if (radios[i].checked) {
			alert_mode = parseInt(radios[i].value);
			break;
		}
	}
	control_display(alert_mode, page_inuse);
}
</script>]]

local alert_js0 = [[control_alert(0);]]
local alert_js1 = [[control_alert(1);]]
local alert_js2 = [[control_alert(2);]]

local p_alert_mode = webpage_cst_param('Ring/Alert Mode',
	webpage_cst_radio('sip.detect.mode',
		{'Monitor "Ring" event on registered SIP extension<br>',
		'None'},
		{'1', '0'}, {alert_js1, alert_js0}))


local page_js0 = [[control_page(0);]]
local page_js1 = [[control_page(1);]]
local p_page_inuse = webpage_cst_param('Page Function',
	webpage_cst_enabled('sip.page.use', {page_js1, page_js0}))

local detect_mode = conf_getpn('sip.detect.mode')
local page_inuse = conf_getpn('sip.page.use')
local hide_alert_ext, hide_alert_auth, hide_alert_event
local base_desc, alert_desc

if detect_mode == 0 then
	hide_alert_ext = 1
	hide_alert_auth = 1
	hide_alert_event = 1
	alert_desc = "Ring"
elseif detect_mode == 1 then
	hide_alert_ext = 0
	hide_alert_auth = 0
	hide_alert_event = 1
	alert_desc = "Ring"
elseif detect_mode == 2 then
	hide_alert_ext = 0
	hide_alert_auth = 1
	hide_alert_event = 0
	alert_desc = "Ring/Alert"
end

if page_inuse == 0 then
	if detect_mode == 2 then
		hide_page = 0
	else
		hide_page = 1
	end
	base_desc = 'SIP'
else
	hide_page = 0
	base_desc = 'Page'
end

local p_base_ext = webpage_cst_param(
	'<div id="base_desc">' .. base_desc .. ' Extension</div>',
	webpage_cst_text('sip.u1.user'), nil, 'sip_ext', hide_page)

local p_base_auth = webpage_cst_param('Authentication ID',
	webpage_cst_text('sip.u1.auth'), nil, 'sip_auth', hide_page)

local p_base_pwd = webpage_cst_pwd('Authentication Password',
	'sip.u1.pwd', 'sip_pwd', hide_page)

local p_alert_ext = webpage_cst_param(
	'<div id="alert_desc">' .. alert_desc .. ' Extension</div>',
	webpage_cst_text('sip.alert1.user'), nil, 'alert_ext', hide_alert_ext)

local p_alert_event = webpage_cst_param('Ring/Alert Events',
	webpage_cst_select('sip.alert1.event',
		{'Ring', 'In-Use', 'Both Ring & In-Use'},
		{'1', '2', '0'}),
	nil, 'alert_event', hide_alert_event)

local p_alert_auth = webpage_cst_param('Authentication ID',
	webpage_cst_text('sip.alert1.auth'), nil, 'alert_auth', hide_alert_auth)

local p_alert_pwd = webpage_cst_pwd('Authentication Password',
	'sip.alert1.pwd', 'alert_pwd', hide_alert_auth)

mg.write(prod_cst_header() ..
	prod_cst_menu('Basic Settings', 'SIP') ..
	webpage_cst_mchd(flags['any'], 0) ..
	form_start .. page_start ..
	secthd_basic .. webpage_subsect_hd ..
	p_proxy ..
	webpage_subsect_ft .. webpage_subsect_hd .. js_control ..
	p_alert_mode .. p_page_inuse ..
	webpage_subsect_ft .. webpage_subsect_hd ..
	p_alert_ext ..  p_alert_event ..
	p_alert_auth .. p_alert_pwd ..
	webpage_subsect_ft .. webpage_subsect_hd ..
	p_base_ext .. p_base_auth .. p_base_pwd ..
	webpage_subsect_ft .. webpage_sect_ft ..
	webpage_page_end .. webpage_form_end_save ..
	webpage_mc_ft ..
	prod_footer)
