-- Copyright (c) 2013-2015 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local flags = {['any'] = 0, ['main'] = 0}

local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)
	conf_update('sip.obproxy', settings, flags, 'main')
	conf_update('sip.stun', settings, flags, 'main')
	conf_update('sip.regexp', settings, flags, 'main')
	conf_update('sip.ka.method', settings, flags, 'main')
	conf_update('sip.ka.period', settings, flags, 'main')
	conf_update('sip.diffport', settings, flags, 'main')

	conf_update('sip.ssr.use', settings, flags, 'main')
	conf_update('sip.bkproxy1', settings, flags, 'main')
	conf_update('sip.bkproxy2', settings, flags, 'main')
	conf_update('sip.ssr.interval', settings, flags, 'main')
	conf_update('sip.ssr.chkact', settings, flags, 'main')
	conf_update('sip.ssr.nofb', settings, flags, 'main')
	conf_update('sip.ssr.method', settings, flags, 'main')

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['main'] == 1 then
		os.execute('/etc/init.d/algo-main reload')
	end
end
session_save()

local form_start = webpage_cst_formstart('/control/sipadv.lua')

local page_start = webpage_cst_pagestart('Advanced SIP Settings')

secthd_adv = webpage_cst_secthd('SIP')

local p_obproxy = webpage_cst_param('Outbound Proxy',
	webpage_cst_text('sip.obproxy'))
local stun_label = [[
<abbr title="Session Traversal Utilities for NAT">STUN</abbr> Server]]
local p_stun = webpage_cst_param(stun_label,
	webpage_cst_text('sip.stun'))
local p_regexp = webpage_cst_param('Register Period (seconds)',
	webpage_cst_text('sip.regexp'))

local js_kam = [[
<script type="text/javascript">
function display_kap(mode) {
	var display = (mode == 0) ? 'none' : 'block';
	document.getElementById("sip_ka_period").style.display = display;
}
</script>]]

local kam_js0 = [[display_kap(0);]]
local kam_js1 = [[display_kap(1);]]
local hide = 1 - conf_getpn('sip.ka.method')
local p_kam = webpage_cst_param('Keep-alive Method',
	webpage_cst_radio('sip.ka.method', {'None', 'Double CRLF'}, nil,
		{kam_js0, kam_js1}))

local p_kap = webpage_cst_param('Keep-alive Period (seconds)',
	webpage_cst_text('sip.ka.period'), nil, 'sip_ka_period', hide)

local p_diffport
if conf_getdevice('feature.diffport') == '1' then
	local diffport_desc = [[Turn this option on for certain proxies, e.g. Cisco Communication Manager 7, to send ring and page SIP requests through different port numbers.]]
	p_diffport = webpage_cst_param('Different Ports for Extensions',
		webpage_cst_enabled('sip.diffport'), diffport_desc)
else
	p_diffport = ''
end

secthd_ssr = webpage_cst_secthd('Server Redundancy')

local js_ssr = [[
<script type="text/javascript">
function display_ssr(mode) {
	var display = (mode == 0) ? 'none' : 'block';
	document.getElementById("ssr_bkproxy1").style.display = display;
	document.getElementById("ssr_bkproxy2").style.display = display;
	document.getElementById("ssr_intv").style.display = display;
	document.getElementById("ssr_chkact").style.display = display;
	document.getElementById("ssr_nofb").style.display = display;
	document.getElementById("ssr_method").style.display = display;
}
</script>]]

local ssr_js0 = [[display_ssr(0);]]
local ssr_js1 = [[display_ssr(1);]]
local hide = 1 - conf_getpn('sip.ssr.use')
local p_ssr_use = webpage_cst_param(
	'Server Redundancy Feature (Multiple SIP Server Support)',
	webpage_cst_enabled('sip.ssr.use', {ssr_js1, ssr_js0}))

local p_bp1 = webpage_cst_param('Backup Server #1',
	webpage_cst_text('sip.bkproxy1'), nil, 'ssr_bkproxy1', hide)
local p_bp2 = webpage_cst_param('Backup Server #2',
	webpage_cst_text('sip.bkproxy2'), nil, 'ssr_bkproxy2', hide)

local ssr_intv_desc = {'120 seconds (2 minutes)', '180 seconds (3 minutes)',
	'300 seconds (5 minutes)', '600 seconds (10 minutes)'}
local ssr_intv_value = {'120', '180', '300', '600'}
local intv_desc = 'Time period between sending monitoring packets to each server. Non-active servers are always polled, and active server may optionally be polled (see below).'
local p_ssr_intv = webpage_cst_param('Polling Interval (seconds)',
	webpage_cst_select('sip.ssr.interval', ssr_intv_desc, ssr_intv_value),
	intv_desc, 'ssr_intv', hide)
local chkact_desc = [[Explicity poll current server to monitor availability. May also be handled automatically by other regular events, so can be disabled to reduce network traffic.]]
local p_chkact = webpage_cst_param('Poll Active Server',
	webpage_cst_enabled('sip.ssr.chkact'), chkact_desc, 'ssr_chkact', hide)
local nofb_desc = [[Reconnect with higher priority server once available, even if backup connection still fine.]]

--reversed logic for nofb
local p_nofb = webpage_cst_param('Automatic Failback',
	webpage_cst_radio('sip.ssr.nofb',{'Enabled', 'Disabled'}, {'0', '1'}),
	nofb_desc, 'ssr_nofb', hide)
local method_desc = [[SIP message used to poll servers to monitor availability.]]
local p_check_method = webpage_cst_param('Polling Method',
	webpage_cst_radio('sip.ssr.method', {'SIP NOTIFY', 'SIP OPTIONS'}),
	method_desc, 'ssr_method', hide)

mg.write(prod_cst_header() ..
	prod_cst_menu('Advanced Settings', 'Advanced SIP') ..
	webpage_cst_mchd(flags['any'], 0) ..
	form_start .. page_start ..
	secthd_adv .. webpage_subsect_hd .. js_kam ..
	p_obproxy .. p_stun .. p_regexp ..
	p_kam .. p_kap .. p_diffport ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_ssr .. webpage_subsect_hd .. js_ssr ..
	p_ssr_use .. p_bp1 .. p_bp2 .. p_ssr_intv .. p_chkact ..
	p_nofb .. p_check_method ..
	webpage_subsect_ft .. webpage_sect_ft ..
	webpage_page_end .. webpage_form_end_save ..
	webpage_mc_ft ..
	prod_footer)
