-- Copyright (c) 2013-2016 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local flags = {['any'] = 0, ['reboot'] = 0}

local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)
	mg.write(settings)
	conf_update('admin.timezone', settings, flags, 'reboot')
	conf_update('net.time1', settings, flags, 'reboot')
	conf_update('net.time2', settings, flags, 'reboot')
	conf_update('net.time3', settings, flags, 'reboot')
	conf_update('net.time4', settings, flags, 'reboot')

	if settings['time.man'] == "1" then
		os.execute('/opt/algo/bin/apply-settings time_manual ' ..
			settings['time.day'] .. ' ' ..
			settings['time.month'] .. ' ' ..
			settings['time.year'] .. ' ' ..
			settings['time.hour'] .. ' ' ..
			settings['time.min'] .. ' ' ..
			settings['time.am'])
		session_save()
	end

	if flags['reboot'] == 1 then
		conf_save()
		os.execute('/opt/algo/bin/apply-settings time')
		g_reboot = 1
		session_save()
	end

end
session_save()

local form_start = webpage_cst_formstart('/control/time.lua')

local page_start = webpage_cst_pagestart('Time Settings')

local secthd_gen = webpage_cst_secthd('General')

local secthd_man = webpage_cst_secthd('Manual Settings', 'man_settings')

function cst_timezone_list(name)
	local timezones = {}

	local file, err = io.open('/opt/algo/cache/timezone.txt', 'r')
	if err then return {} end
	while true do
		local line = file:read()
		if line == nil then break end
		table.insert(timezones, line)
	end
	return webpage_cst_select(name, timezones, timezones)
end

local js_show = [[
<script type="text/javascript">
function read_radio_value(radios)
{
	var val = 0;

	for (var i = 0, length = radios.length; i < length; i++) {
		if (radios[i].checked) {
			val = parseInt(radios[i].value);
			break;
		}
	}
	return val;
}

function show_page() {
	var mode;
	mode = read_radio_value(document.getElementsByName('time.man'));

	var display = (mode == 0) ? 'none' : 'block';
	document.getElementById('man_settings').style.display = display;
}

// set default to automatic
document.getElementsByName('time.man')[0].checked = true;

</script>]]

local man_time = [[show_page();]]

local p_mode_sel = webpage_cst_param('Set Time and Date Manually',
	webpage_cst_radio('time.man', {"Automatic", "Manual"}, {'0','1'}, {man_time, man_time}),
	'Manually set time and date settings are temporary.  Upon rebooting the device, the current time will be fetched from the NTP servers above.')

local p_timezone = webpage_cst_param('Timezone',
	cst_timezone_list('admin.timezone'))

local p_ntp1 = webpage_cst_param('NTP Time Server 1',
	webpage_cst_text('net.time1'))
local p_ntp2 = webpage_cst_param('NTP Time Server 2',
	webpage_cst_text('net.time2'))
local p_ntp3 = webpage_cst_param('NTP Time Server 3',
	webpage_cst_text('net.time3'))
local p_ntp4 = webpage_cst_param('NTP Time Server 4',
	webpage_cst_text('net.time4'))

local days = {"1","2","3","4","5","6","7","8","9",
		 "10","11","12","13","14","15","16","17","18","19",
		 "20","21","22","23","24","25","26","27","28","29",
		 "30","31"}
local day_vals = {"01","02","03","04","05","06","07","08","09",
		 "10","11","12","13","14","15","16","17","18","19",
		 "20","21","22","23","24","25","26","27","28","29",
		 "30","31"}

local months = {"January", "February", "March", "April",
		"May", "June", "July", "August", "September",
		"October", "November", "December"}
local month_vals = {"01","02","03","04","05","06",
					"07","08","09","10","11","12"}

local p_day = webpage_cst_param('Day',
	webpage_cst_select('time.day', days, day_vals, nil, nil, nil, os.date("%d")))

local p_month = webpage_cst_param('Month',
	webpage_cst_select('time.month', months, month_vals, nil, nil, nil, os.date("%m")))

local p_year = webpage_cst_param('Year',
	webpage_cst_text('time.year', nil,  os.date("%Y")))

local hours = {"1","2","3","4","5","6",
				"7","8","9","10","11","12"}

local hour_vals = {"01","02","03","04","05","06",
				"07","08","09","10","11","12"}

local minutes = {"00","01","02","03","04","05","06","07","08","09",
		 "10","11","12","13","14","15","16","17","18","19",
		 "20","21","22","23","24","25","26","27","28","29",
		 "30","31","32","33","34","35","36","37","38","39",
		 "40","41","42","43","44","45","46","47","48","49",
		 "50","51","52","53","54","55","56","57","58","59"}

local p_time = webpage_cst_param('Time',
	webpage_cst_select('time.hour', hours, hour_vals, nil, nil, nil, os.date("%I")) ..
	webpage_cst_select('time.min', minutes, minutes, nil, nil, nil, os.date("%M")) ..
	webpage_cst_radio('time.am', {"AM","PM"}, {"AM","PM"}, nil, nil, nil, nil, nil, os.date("%p")))

local js_showall = [[<script type="text/javascript">
show_page();
</script>]]

mg.write(prod_cst_header() ..
	prod_cst_menu('Advanced Settings', 'Time') ..
	webpage_cst_mchd(flags['reboot'], flags['reboot']) ..
	form_start .. page_start ..
	secthd_gen .. webpage_subsect_hd ..
	p_timezone .. p_ntp1 .. p_ntp2 .. p_ntp3 .. p_ntp4 ..
	p_mode_sel .. js_show ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_man .. webpage_subsect_hd ..
	p_day .. p_month .. p_year ..
	p_time ..
	webpage_subsect_ft .. webpage_sect_ft ..
	webpage_page_end .. webpage_form_end_save ..
	js_showall ..
	webpage_mc_ft ..
	prod_footer)
