-- Copyright (c) 2013-2015 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')

conf_load()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end

local js_selected_tone = 'var selected_tone = "";'
local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local msg = {}
	local vars = {}
	local cmd

	webpage_parse_multipart(cl, vars)

	if vars['tonefile'] then
		if vars['toneman'] == 'Play' then
			cmd = '/opt/algo/bin/toneman-play ' .. vars['tonefile']
			js_selected_tone = string.format(
				'var selected_tone = "%s";',
				vars['tonefile'])
		elseif vars['toneman'] == 'Delete' then
			cmd = '/opt/algo/bin/toneman-delete ' ..
				vars['tonefile']
		elseif vars['toneman'] == 'Rename' then
			if string.find(vars['newname'], '%s') then
				msg['Rename Failed'] = 'Not a valid file name: please remove spaces.'
			elseif vars['newname'] then
				cmd = '/opt/algo/bin/toneman-rename ' ..
					vars['tonefile'] .. ' ' ..
					vars['newname']
				js_selected_tone = string.format(
					'var selected_tone = "%s";',
					vars['newname'])
			end
		elseif vars['toneman'] == 'Auto Sound Level' then
			cmd = '/opt/algo/bin/toneman-autosnd ' ..
				vars['tonefile']
			js_selected_tone = string.format(
				'var selected_tone = "%s";',
				vars['tonefile'])
		end
	end

	if vars['upload'] then
		cmd = '/opt/algo/bin/toneman-upload'
	end

	if cmd then
		os.execute(cmd)
	end
end
session_save()

local page_start = webpage_cst_pagestart('Tone Management')

local secthd_upload = webpage_cst_secthd('Upload')

local form_upload = [[<form method="post" enctype="multipart/form-data" action="/control/toneman.lua"  onsubmit="return validate_fn(document.getElementsByName('newtone')[0].value, 'Not a valid file extension: supports wav and zip only.');">]]

local wave_desc = [[8kHz/16kHz, 16-bit, Mono, PCM/u-law WAV File, or such files in zip format.  Please limit the file name to 32 characters, and no spaces.]]
local p_upload = webpage_cst_param('New Tone File',
	webpage_cst_upload('newtone') ..
	webpage_cst_button('upload', 'Upload', 'upload'),
	wave_desc)

local form_mgmt = webpage_cst_formstart('/control/toneman.lua')

local secthd_tone = webpage_cst_secthd('Tone Files', 'sect_tones')

local js_act = [[<script type="text/javascript">
var rename_hidden = 1
function onclick_rename()
{
	if (rename_hidden) {
		rename_hidden = 0;
		document.getElementById("subsect_rename").style.display =
			"block";
		document.getElementsByName("download")[0].disabled = true;
		var ctrls = document.getElementsByName('toneman');
		for (var i = 0, length = ctrls.length; i < length; i++) {
			if (ctrls[i].value != "Rename")
				ctrls[i].disabled = true;
		}
	} else {
		rename_hidden = 1;
		document.getElementById("subsect_rename").style.display =
			"none";
		document.getElementsByName("download")[0].disabled = false;
		var ctrls = document.getElementsByName('toneman');
		for (var i = 0, length = ctrls.length; i < length; i++) {
			if (ctrls[i].value != "Rename")
				ctrls[i].disabled = false;
		}
	}
}

function find_selected()
{
	radios = document.getElementsByName('tonefile');
	for (var i = 0, length = radios.length; i < length; i++) {
		if (radios[i].checked) {
			return radios[i].value;
		}
	}
	return null;
}

function validate_fn(fullfn, errext)
{
	var re = new RegExp(/([^\/\\]+)$/);
	var fn = re.exec(fullfn);
	if (fn == null) {
		alert('Not a valid file name.');
		return false;
	}
	var ext = fn[0].substring(fn[0].lastIndexOf('.') + 1);
	if (ext != "wav" && ext != "zip") {
		alert(errext);
		return false;
	}
	if (fn[0].indexOf(' ') >= 0) {
		alert('Not a valid file name: please remove spaces.');
		return false;
	}
	return true;
}

function validate_delete()
{
	var selected = find_selected();
	ret = validate_fn(selected, "Please select a tone file.");
	if (! ret)
		return ret;

	return confirm("Do you want to delete file?\n" + selected);
}

function validate_rename()
{
	var selected = find_selected();
	ret = validate_fn(selected, "Please select a tone file.");
	if (! ret)
		return ret;

	var newname = document.getElementsByName('newname')[0].value;
	ret = validate_fn(newname,
		'Not a valid file extension: supports wav only.')
	if (! ret)
		return ret;

	return confirm("Do you want to rename tone file?\n" +
		selected + " => " + newname);
}

function validate_play()
{
	var selected = find_selected();
	return validate_fn(selected, "Please select a tone file.");
}

function validate_autosnd()
{
	var selected = find_selected();
	ret = validate_fn(selected, "Please select a tone file.");
	if (! ret)
		return ret;

	return confirm("Do you want to generate Auto Sound Level tone file based on the selected one?\n" + selected);
}
]] .. js_selected_tone .. [[
</script>]]

local p_act = webpage_cst_param('Functions',
	webpage_cst_button('toneman', 'Delete', 'delete',
		'return validate_delete()') ..
	webpage_cst_button('rename', 'Rename', 'rename',
		'onclick_rename()', 1) ..
	webpage_cst_button('toneman', 'Play', 'play',
		'return validate_play()') ..
	webpage_cst_button('toneman', 'Auto Sound Level', 'autosnd',
		'return validate_autosnd()'),
	'Please select a file from the list below to use these functions.')

local subsecthd_rename = webpage_cst_subsect_hd('subsect_rename', 1)

local p_rename = webpage_cst_param('Rename to',
	webpage_cst_text('newname') ..
	webpage_cst_button('toneman', 'Rename', 'rename',
		'return validate_rename()'))

mg.write(prod_cst_header() ..
	prod_cst_menu('Advanced Settings', 'Tones') ..
	webpage_cst_mchd(0, 0) .. page_start ..
	secthd_upload .. webpage_subsect_hd .. form_upload ..
	p_upload ..
	webpage_subsect_ft .. webpage_sect_ft .. webpage_form_end ..
	form_mgmt ..
	secthd_tone .. webpage_subsect_hd ..  js_act ..
	p_act .. webpage_subsect_ft ..
	subsecthd_rename .. p_rename ..
	webpage_subsect_ft .. webpage_subsect_hd)

mg.write([[<table class="cbi-section-table" style="width:100%">
<tr class="cbi-section-table-titles">
<th class="cbi-setion-table-cell" style="width: 5%">&nbsp;</th>
<th class="cbi-setion-table-cell">Name</th>
<th class="cbi-setion-table-cell">Size</th>
<th class="cbi-setion-table-cell">Modification Date/Time</th>
</tr>]])

local allfiles = {}
local path = '/opt/algo/web/tones'
for file in lfs.dir(path) do
	if file ~= '.' and file ~= '..' then
		table.insert(allfiles, file)
	end
end
table.sort(allfiles)

local total = 0
local rowstyle = 1
local attr
local size
local mod
for i, fn in ipairs(allfiles) do
	attr = lfs.attributes(path .. '/' .. fn)
	size = attr['size']
	total = total + size
	size = math.floor(tonumber(size) / 1024)
	mod = os.date('%b %d, %Y %H:%M', attr['modification'])
	mg.write(string.format([[
<tr class="cbi-section-table-row cbi-rowstyle-%d">
<td><input type="radio" name="tonefile" value="%s" /></td>
<td style="text-align:left"><a href="/tones/%s">%s</a></td>
<td style="text-align:right">%d kB</td>
<td style="text-align:right">%s</td>
</tr>]], rowstyle, fn, fn, fn, size, mod))
	if rowstyle == 1 then rowstyle = 2 else rowstyle = 1 end
end
total = math.floor(tonumber(total) / 1024)

local p_cnt = webpage_cst_param('Number of Files', #allfiles)
local p_total = webpage_cst_param('Total Size', total .. ' kB')

if rowstyle == 1 then
	mg.write('<tr class="cbi-section-table-row cbi-rowstyle-1"><td>&nbsp;</td><td></td><td></td><td></td></tr>')
end

mg.write([[<tr class="cbi-section-table-row cbi-rowstyle-2">
<td></td><td style="text-align:left"><span class="cbi-value-helpicon">
<img src="/static/information-white.png" alt="help"></span>If you wish to copy a tone file to your local computer, right click the name to download.</td>
<td></td><td></td></tr>]])

local sum_tmpl = [[<tr class="cbi-section-table-row cbi-rowstyle-2">
<td colspan="2" style="text-align:right">%s</td><td style="text-align:right">%s</td><td></td></tr>]]

mg.write(string.format(sum_tmpl, 'Number of Files', #allfiles))
mg.write(string.format(sum_tmpl, 'Total Size', total .. ' kB'))

local js_initval = [[<script type="text/javascript">
if (selected_tone) {
	var tones = document.getElementsByName('tonefile');
	for (var i = 0, length = tones.length; i < length; i++) {
		if (tones[i].value == selected_tone) {
			tones[i].checked = true;
			break;
		}
	}
} else {
	document.getElementsByName("tonefile")[0].checked = true;
}
</script>]]

mg.write('</table>' .. js_initval ..
	webpage_subsect_ft .. webpage_sect_ft .. webpage_form_end ..
	webpage_page_end ..
	webpage_mc_ft ..
	prod_footer)
